/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.DataConstants;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.data.NotesDatabaseHelper.TABLE;
import net.micode.notes.gtask.exception.ActionFailureException;

import org.json.JSONException;
import org.json.JSONObject;


public class SqlData {
    // 定义一个常量，用于标识类的简单名称
    private static final String TAG = SqlData.class.getSimpleName();

    // 定义一个常量，用于标识无效的ID
    private static final int INVALID_ID = -99999;

    // 定义一个常量，用于存储数据的列名
    public static final String[] PROJECTION_DATA = new String[] {
            DataColumns.ID, DataColumns.MIME_TYPE, DataColumns.CONTENT, DataColumns.DATA1,
            DataColumns.DATA3
    };

    // 定义一个常量，用于标识数据的ID列
    public static final int DATA_ID_COLUMN = 0;

    // 定义一个常量，用于标识数据的MIME类型列
    public static final int DATA_MIME_TYPE_COLUMN = 1;

    // 定义一个常量，用于标识数据的CONTENT列
    public static final int DATA_CONTENT_COLUMN = 2;

    // 定义一个常量，用于标识数据的DATA1列
    public static final int DATA_CONTENT_DATA_1_COLUMN = 3;

    // 定义一个常量，用于标识数据的DATA3列
    public static final int DATA_CONTENT_DATA_3_COLUMN = 4;

    // 定义一个ContentResolver对象，用于访问内容提供者
    private ContentResolver mContentResolver;

    // 定义一个布尔值，用于标识是否创建
    private boolean mIsCreate;

    // 定义一个长整型变量，用于存储数据的ID
    private long mDataId;

    // 定义一个字符串变量，用于存储数据的MIME类型
    private String mDataMimeType;

    // 定义一个字符串变量，用于存储数据的CONTENT
    private String mDataContent;

    // 定义一个长整型变量，用于存储数据的DATA1
    private long mDataContentData1;

    // 定义一个字符串变量，用于存储数据的DATA3
    private String mDataContentData3;

    // 定义一个ContentValues对象，用于存储数据的差异
    private ContentValues mDiffDataValues;

    // 构造函数，用于创建一个新的SqlData对象
    public SqlData(Context context) {
        mContentResolver = context.getContentResolver();
        mIsCreate = true;
        mDataId = INVALID_ID;
        mDataMimeType = DataConstants.NOTE;
        mDataContent = "";
        mDataContentData1 = 0;
        mDataContentData3 = "";
        mDiffDataValues = new ContentValues();
    }

    // 构造函数，用于从Cursor中加载数据
    public SqlData(Context context, Cursor c) {
        mContentResolver = context.getContentResolver();
        mIsCreate = false;
        loadFromCursor(c);
        mDiffDataValues = new ContentValues();
    }

    // 从Cursor中加载数据
    private void loadFromCursor(Cursor c) {
        mDataId = c.getLong(DATA_ID_COLUMN);
        mDataMimeType = c.getString(DATA_MIME_TYPE_COLUMN);
        mDataContent = c.getString(DATA_CONTENT_COLUMN);
        mDataContentData1 = c.getLong(DATA_CONTENT_DATA_1_COLUMN);
        mDataContentData3 = c.getString(DATA_CONTENT_DATA_3_COLUMN);
    }

    // 设置数据
    public void setContent(JSONObject js) throws JSONException {
        long dataId = js.has(DataColumns.ID) ? js.getLong(DataColumns.ID) : INVALID_ID;
        if (mIsCreate || mDataId != dataId) {
            mDiffDataValues.put(DataColumns.ID, dataId);
        }
        mDataId = dataId;

        String dataMimeType = js.has(DataColumns.MIME_TYPE) ? js.getString(DataColumns.MIME_TYPE)
                : DataConstants.NOTE;
        if (mIsCreate || !mDataMimeType.equals(dataMimeType)) {
            mDiffDataValues.put(DataColumns.MIME_TYPE, dataMimeType);
        }
        mDataMimeType = dataMimeType;

        String dataContent = js.has(DataColumns.CONTENT) ? js.getString(DataColumns.CONTENT) : "";
        if (mIsCreate || !mDataContent.equals(dataContent)) {
            mDiffDataValues.put(DataColumns.CONTENT, dataContent);
        }
        mDataContent = dataContent;

        long dataContentData1 = js.has(DataColumns.DATA1) ? js.getLong(DataColumns.DATA1) : 0;
        if (mIsCreate || mDataContentData1 != dataContentData1) {
            mDiffDataValues.put(DataColumns.DATA1, dataContentData1);
        }
        mDataContentData1 = dataContentData1;

        String dataContentData3 = js.has(DataColumns.DATA3) ? js.getString(DataColumns.DATA3) : "";
        if (mIsCreate || !mDataContentData3.equals(dataContentData3)) {
            mDiffDataValues.put(DataColumns.DATA3, dataContentData3);
        }
        mDataContentData3 = dataContentData3;
    }

    // 获取数据
    public JSONObject getContent() throws JSONException {
        if (mIsCreate) {
            Log.e(TAG, "it seems that we haven't created this in database yet");
            return null;
        }
        JSONObject js = new JSONObject();
        js.put(DataColumns.ID, mDataId);
        js.put(DataColumns.MIME_TYPE, mDataMimeType);
        js.put(DataColumns.CONTENT, mDataContent);
        js.put(DataColumns.DATA1, mDataContentData1);
        js.put(DataColumns.DATA3, mDataContentData3);
        return js;
    }

    // 提交数据
    // 提交数据
    public void commit(long noteId, boolean validateVersion, long version) {

        // 如果是创建
        if (mIsCreate) {
            // 如果数据ID无效且数据值中包含ID列
            if (mDataId == INVALID_ID && mDiffDataValues.containsKey(DataColumns.ID)) {
                // 移除ID列
                mDiffDataValues.remove(DataColumns.ID);
            }

            // 添加NOTE_ID列
            mDiffDataValues.put(DataColumns.NOTE_ID, noteId);
            // 插入数据
            Uri uri = mContentResolver.insert(Notes.CONTENT_DATA_URI, mDiffDataValues);
            try {
                // 获取数据ID
                mDataId = Long.valueOf(uri.getPathSegments().get(1));
            } catch (NumberFormatException e) {
                // 打印错误日志
                Log.e(TAG, "Get note id error :" + e.toString());
                // 抛出异常
                throw new ActionFailureException("create note failed");
            }
        } else {
            // 如果数据值不为空
            if (mDiffDataValues.size() > 0) {
                // 更新结果
                int result = 0;
                // 如果不验证版本
                if (!validateVersion) {
                    // 更新数据
                    result = mContentResolver.update(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues, null, null);
                } else {
                    // 更新数据，验证版本
                    result = mContentResolver.update(ContentUris.withAppendedId(
                            Notes.CONTENT_DATA_URI, mDataId), mDiffDataValues,
                            " ? in (SELECT " + NoteColumns.ID + " FROM " + TABLE.NOTE
                                    + " WHERE " + NoteColumns.VERSION + "=?)", new String[] {
                                    String.valueOf(noteId), String.valueOf(version)
                            });
                }
                // 如果更新结果为0
                if (result == 0) {
                    // 打印警告日志
                    Log.w(TAG, "there is no update. maybe user updates note when syncing");
                }
            }
        }

        // 清空数据值
        mDiffDataValues.clear();
        // 设置为非创建
        mIsCreate = false;
    }

    // 获取数据的ID
    public long getId() {
        return mDataId;
    }
}
