/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.data;

import android.appwidget.AppWidgetManager;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.DataColumns;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.gtask.exception.ActionFailureException;
import net.micode.notes.tool.GTaskStringUtils;
import net.micode.notes.tool.ResourceParser;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;


public class SqlNote {
    private static final String TAG = SqlNote.class.getSimpleName();

    private static final int INVALID_ID = -99999;

    public static final String[] PROJECTION_NOTE = new String[] {
            NoteColumns.ID, NoteColumns.ALERTED_DATE, NoteColumns.BG_COLOR_ID,
            NoteColumns.CREATED_DATE, NoteColumns.HAS_ATTACHMENT, NoteColumns.MODIFIED_DATE,
            NoteColumns.NOTES_COUNT, NoteColumns.PARENT_ID, NoteColumns.SNIPPET, NoteColumns.TYPE,
            NoteColumns.WIDGET_ID, NoteColumns.WIDGET_TYPE, NoteColumns.SYNC_ID,
            NoteColumns.LOCAL_MODIFIED, NoteColumns.ORIGIN_PARENT_ID, NoteColumns.GTASK_ID,
            NoteColumns.VERSION
    };

    public static final int ID_COLUMN = 0;

    public static final int ALERTED_DATE_COLUMN = 1;

    public static final int BG_COLOR_ID_COLUMN = 2;

    public static final int CREATED_DATE_COLUMN = 3;

    public static final int HAS_ATTACHMENT_COLUMN = 4;

    public static final int MODIFIED_DATE_COLUMN = 5;

    public static final int NOTES_COUNT_COLUMN = 6;

    public static final int PARENT_ID_COLUMN = 7;

    public static final int SNIPPET_COLUMN = 8;

    public static final int TYPE_COLUMN = 9;

    public static final int WIDGET_ID_COLUMN = 10;

    public static final int WIDGET_TYPE_COLUMN = 11;

    public static final int SYNC_ID_COLUMN = 12;

    public static final int LOCAL_MODIFIED_COLUMN = 13;

    public static final int ORIGIN_PARENT_ID_COLUMN = 14;

    public static final int GTASK_ID_COLUMN = 15;

    public static final int VERSION_COLUMN = 16;

    private Context mContext;

    private ContentResolver mContentResolver;

    private boolean mIsCreate;

    private long mId;

    private long mAlertDate;

    private int mBgColorId;

    private long mCreatedDate;

    private int mHasAttachment;

    private long mModifiedDate;

    private long mParentId;

    private String mSnippet;

    private int mType;

    private int mWidgetId;

    private int mWidgetType;

    private long mOriginParent;

    private long mVersion;

    private ContentValues mDiffNoteValues;

    private ArrayList<SqlData> mDataList;

    // 构造函数，初始化SqlNote对象
    public SqlNote(Context context) {
        // 保存上下文
        mContext = context;
        // 获取ContentResolver
        mContentResolver = context.getContentResolver();
        // 标记为创建
        mIsCreate = true;
        // 初始化ID为无效ID
        mId = INVALID_ID;
        // 初始化提醒日期为0
        mAlertDate = 0;
        // 获取默认背景颜色ID
        mBgColorId = ResourceParser.getDefaultBgId(context);
        // 获取当前时间作为创建日期
        mCreatedDate = System.currentTimeMillis();
        // 初始化是否有附件为0
        mHasAttachment = 0;
        // 获取当前时间作为修改日期
        mModifiedDate = System.currentTimeMillis();
        // 初始化父ID为0
        mParentId = 0;
        // 初始化片段为空字符串
        mSnippet = "";
        // 初始化类型为笔记
        mType = Notes.TYPE_NOTE;
        // 初始化小部件ID为无效ID
        mWidgetId = AppWidgetManager.INVALID_APPWIDGET_ID;
        // 初始化小部件类型为无效类型
        mWidgetType = Notes.TYPE_WIDGET_INVALIDE;
        // 初始化原始父ID为0
        mOriginParent = 0;
        // 初始化版本号为0
        mVersion = 0;
        // 初始化ContentValues对象
        mDiffNoteValues = new ContentValues();
        // 初始化数据列表
        mDataList = new ArrayList<SqlData>();
    }

    // 构造函数，用于从Cursor中加载SqlNote对象
    public SqlNote(Context context, Cursor c) {
        // 保存上下文
        mContext = context;
        // 获取ContentResolver
        mContentResolver = context.getContentResolver();
        // 标记为未创建
        mIsCreate = false;
        // 从Cursor中加载SqlNote对象
        loadFromCursor(c);
        // 初始化数据列表
        mDataList = new ArrayList<SqlData>();
        // 如果类型为Notes.TYPE_NOTE，则加载内容
        if (mType == Notes.TYPE_NOTE)
            loadDataContent();
        // 初始化ContentValues
        mDiffNoteValues = new ContentValues();
    }

    // 构造函数，传入上下文和id
    public SqlNote(Context context, long id) {
        // 保存上下文
        mContext = context;
        // 获取内容解析器
        mContentResolver = context.getContentResolver();
        // 标记为未创建
        mIsCreate = false;
        // 从游标中加载
        loadFromCursor(id);
        // 创建数据列表
        mDataList = new ArrayList<SqlData>();
        // 如果类型为笔记，则加载内容
        if (mType == Notes.TYPE_NOTE)
            loadDataContent();
        // 创建内容值
        mDiffNoteValues = new ContentValues();

    }

    // 从游标中加载数据
    private void loadFromCursor(long id) {
        // 声明游标变量
        Cursor c = null;
        try {
            // 从ContentResolver中查询数据
            c = mContentResolver.query(Notes.CONTENT_NOTE_URI, PROJECTION_NOTE, "(_id=?)",
                    new String[] {
                        String.valueOf(id)
                    }, null);
            // 如果游标不为空
            if (c != null) {
                // 将游标移动到下一行
                c.moveToNext();
                // 从游标中加载数据
                loadFromCursor(c);
            } else {
                // 如果游标为空，则输出警告日志
                Log.w(TAG, "loadFromCursor: cursor = null");
            }
        } finally {
            // 关闭游标
            if (c != null)
                c.close();
        }
    }

    // 从游标中加载数据
    private void loadFromCursor(Cursor c) {
        // 获取ID
        mId = c.getLong(ID_COLUMN);
        // 获取提醒日期
        mAlertDate = c.getLong(ALERTED_DATE_COLUMN);
        // 获取背景颜色ID
        mBgColorId = c.getInt(BG_COLOR_ID_COLUMN);
        // 获取创建日期
        mCreatedDate = c.getLong(CREATED_DATE_COLUMN);
        // 获取是否有附件
        mHasAttachment = c.getInt(HAS_ATTACHMENT_COLUMN);
        // 获取修改日期
        mModifiedDate = c.getLong(MODIFIED_DATE_COLUMN);
        // 获取父ID
        mParentId = c.getLong(PARENT_ID_COLUMN);
        // 获取片段
        mSnippet = c.getString(SNIPPET_COLUMN);
        // 获取类型
        mType = c.getInt(TYPE_COLUMN);
        // 获取小部件ID
        mWidgetId = c.getInt(WIDGET_ID_COLUMN);
        // 获取小部件类型
        mWidgetType = c.getInt(WIDGET_TYPE_COLUMN);
        // 获取版本
        mVersion = c.getLong(VERSION_COLUMN);
    }

    // 加载数据内容
    private void loadDataContent() {
        // 声明游标
        Cursor c = null;
        // 清空数据列表
        mDataList.clear();
        try {
            // 查询数据
            c = mContentResolver.query(Notes.CONTENT_DATA_URI, SqlData.PROJECTION_DATA,
                    "(note_id=?)", new String[] {
                        String.valueOf(mId)
                    }, null);
            // 如果游标不为空
            if (c != null) {
                // 如果游标中没有数据
                if (c.getCount() == 0) {
                    // 打印警告日志
                    Log.w(TAG, "it seems that the note has not data");
                    return;
                }
                // 遍历游标
                while (c.moveToNext()) {
                    // 创建SqlData对象
                    SqlData data = new SqlData(mContext, c);
                    // 将数据添加到数据列表中
                    mDataList.add(data);
                }
            } else {
                // 打印警告日志
                Log.w(TAG, "loadDataContent: cursor = null");
            }
        } finally {
            // 关闭游标
            if (c != null)
                c.close();
        }
    }

    public boolean setContent(JSONObject js) {
        try {
            // 获取note对象
            JSONObject note = js.getJSONObject(GTaskStringUtils.META_HEAD_NOTE);
            // 如果note的类型是系统文件夹，则不能设置
            if (note.getInt(NoteColumns.TYPE) == Notes.TYPE_SYSTEM) {
                Log.w(TAG, "cannot set system folder");
            } else if (note.getInt(NoteColumns.TYPE) == Notes.TYPE_FOLDER) {
                // for folder we can only update the snnipet and type
                String snippet = note.has(NoteColumns.SNIPPET) ? note
                        .getString(NoteColumns.SNIPPET) : "";
                if (mIsCreate || !mSnippet.equals(snippet)) {
                    mDiffNoteValues.put(NoteColumns.SNIPPET, snippet);
                }
                mSnippet = snippet;

                int type = note.has(NoteColumns.TYPE) ? note.getInt(NoteColumns.TYPE)
                        : Notes.TYPE_NOTE;
                if (mIsCreate || mType != type) {
                    mDiffNoteValues.put(NoteColumns.TYPE, type);
                }
                mType = type;
            } else if (note.getInt(NoteColumns.TYPE) == Notes.TYPE_NOTE) {
                JSONArray dataArray = js.getJSONArray(GTaskStringUtils.META_HEAD_DATA);
                long id = note.has(NoteColumns.ID) ? note.getLong(NoteColumns.ID) : INVALID_ID;
                if (mIsCreate || mId != id) {
                    mDiffNoteValues.put(NoteColumns.ID, id);
                }
                mId = id;

                long alertDate = note.has(NoteColumns.ALERTED_DATE) ? note
                        .getLong(NoteColumns.ALERTED_DATE) : 0;
                if (mIsCreate || mAlertDate != alertDate) {
                    mDiffNoteValues.put(NoteColumns.ALERTED_DATE, alertDate);
                }
                mAlertDate = alertDate;

                int bgColorId = note.has(NoteColumns.BG_COLOR_ID) ? note
                        .getInt(NoteColumns.BG_COLOR_ID) : ResourceParser.getDefaultBgId(mContext);
                if (mIsCreate || mBgColorId != bgColorId) {
                    mDiffNoteValues.put(NoteColumns.BG_COLOR_ID, bgColorId);
                }
                mBgColorId = bgColorId;

                long createDate = note.has(NoteColumns.CREATED_DATE) ? note
                        .getLong(NoteColumns.CREATED_DATE) : System.currentTimeMillis();
                if (mIsCreate || mCreatedDate != createDate) {
                    mDiffNoteValues.put(NoteColumns.CREATED_DATE, createDate);
                }
                mCreatedDate = createDate;

                int hasAttachment = note.has(NoteColumns.HAS_ATTACHMENT) ? note
                        .getInt(NoteColumns.HAS_ATTACHMENT) : 0;
                if (mIsCreate || mHasAttachment != hasAttachment) {
                    mDiffNoteValues.put(NoteColumns.HAS_ATTACHMENT, hasAttachment);
                }
                mHasAttachment = hasAttachment;

                long modifiedDate = note.has(NoteColumns.MODIFIED_DATE) ? note
                        .getLong(NoteColumns.MODIFIED_DATE) : System.currentTimeMillis();
                if (mIsCreate || mModifiedDate != modifiedDate) {
                    mDiffNoteValues.put(NoteColumns.MODIFIED_DATE, modifiedDate);
                }
                mModifiedDate = modifiedDate;

                long parentId = note.has(NoteColumns.PARENT_ID) ? note
                        .getLong(NoteColumns.PARENT_ID) : 0;
                if (mIsCreate || mParentId != parentId) {
                    mDiffNoteValues.put(NoteColumns.PARENT_ID, parentId);
                }
                mParentId = parentId;

                String snippet = note.has(NoteColumns.SNIPPET) ? note
                        .getString(NoteColumns.SNIPPET) : "";
                if (mIsCreate || !mSnippet.equals(snippet)) {
                    mDiffNoteValues.put(NoteColumns.SNIPPET, snippet);
                }
                mSnippet = snippet;

                int type = note.has(NoteColumns.TYPE) ? note.getInt(NoteColumns.TYPE)
                        : Notes.TYPE_NOTE;
                if (mIsCreate || mType != type) {
                    mDiffNoteValues.put(NoteColumns.TYPE, type);
                }
                mType = type;

                int widgetId = note.has(NoteColumns.WIDGET_ID) ? note.getInt(NoteColumns.WIDGET_ID)
                        : AppWidgetManager.INVALID_APPWIDGET_ID;
                if (mIsCreate || mWidgetId != widgetId) {
                    mDiffNoteValues.put(NoteColumns.WIDGET_ID, widgetId);
                }
                mWidgetId = widgetId;

                int widgetType = note.has(NoteColumns.WIDGET_TYPE) ? note
                        .getInt(NoteColumns.WIDGET_TYPE) : Notes.TYPE_WIDGET_INVALIDE;
                if (mIsCreate || mWidgetType != widgetType) {
                    mDiffNoteValues.put(NoteColumns.WIDGET_TYPE, widgetType);
                }
                mWidgetType = widgetType;

                long originParent = note.has(NoteColumns.ORIGIN_PARENT_ID) ? note
                        .getLong(NoteColumns.ORIGIN_PARENT_ID) : 0;
                if (mIsCreate || mOriginParent != originParent) {
                    mDiffNoteValues.put(NoteColumns.ORIGIN_PARENT_ID, originParent);
                }
                mOriginParent = originParent;

                for (int i = 0; i < dataArray.length(); i++) {
                    JSONObject data = dataArray.getJSONObject(i);
                    SqlData sqlData = null;
                    if (data.has(DataColumns.ID)) {
                        long dataId = data.getLong(DataColumns.ID);
                        for (SqlData temp : mDataList) {
                            if (dataId == temp.getId()) {
                                sqlData = temp;
                            }
                        }
                    }

                    if (sqlData == null) {
                        sqlData = new SqlData(mContext);
                        mDataList.add(sqlData);
                    }

                    sqlData.setContent(data);
                }
            }
        } catch (JSONException e) {
            Log.e(TAG, e.toString());
            e.printStackTrace();
            return false;
        }
        return true;
    }

    // 获取内容
    public JSONObject getContent() {
        try {
            // 创建一个JSONObject对象
            JSONObject js = new JSONObject();

            // 如果mIsCreate为true，说明还没有在数据库中创建
            if (mIsCreate) {
                Log.e(TAG, "it seems that we haven't created this in database yet");
                return null;
            }

            // 创建一个JSONObject对象
            JSONObject note = new JSONObject();
            // 如果mType为Notes.TYPE_NOTE
            if (mType == Notes.TYPE_NOTE) {
                // 将mId放入note中
                note.put(NoteColumns.ID, mId);
                // 将mAlertDate放入note中
                note.put(NoteColumns.ALERTED_DATE, mAlertDate);
                // 将mBgColorId放入note中
                note.put(NoteColumns.BG_COLOR_ID, mBgColorId);
                // 将mCreatedDate放入note中
                note.put(NoteColumns.CREATED_DATE, mCreatedDate);
                // 将mHasAttachment放入note中
                note.put(NoteColumns.HAS_ATTACHMENT, mHasAttachment);
                // 将mModifiedDate放入note中
                note.put(NoteColumns.MODIFIED_DATE, mModifiedDate);
                // 将mParentId放入note中
                note.put(NoteColumns.PARENT_ID, mParentId);
                // 将mSnippet放入note中
                note.put(NoteColumns.SNIPPET, mSnippet);
                // 将mType放入note中
                note.put(NoteColumns.TYPE, mType);
                // 将mWidgetId放入note中
                note.put(NoteColumns.WIDGET_ID, mWidgetId);
                // 将mWidgetType放入note中
                note.put(NoteColumns.WIDGET_TYPE, mWidgetType);
                // 将mOriginParent放入note中
                note.put(NoteColumns.ORIGIN_PARENT_ID, mOriginParent);
                // 将note放入js中
                js.put(GTaskStringUtils.META_HEAD_NOTE, note);

                // 创建一个JSONArray对象
                JSONArray dataArray = new JSONArray();
                // 遍历mDataList
                for (SqlData sqlData : mDataList) {
                    // 获取sqlData的内容
                    JSONObject data = sqlData.getContent();
                    // 如果data不为空
                    if (data != null) {
                        // 将data放入dataArray中
                        dataArray.put(data);
                    }
                }
                // 将dataArray放入js中
                js.put(GTaskStringUtils.META_HEAD_DATA, dataArray);
            // 如果mType为Notes.TYPE_FOLDER或Notes.TYPE_SYSTEM
            } else if (mType == Notes.TYPE_FOLDER || mType == Notes.TYPE_SYSTEM) {
                // 将mId放入note中
                note.put(NoteColumns.ID, mId);
                // 将mType放入note中
                note.put(NoteColumns.TYPE, mType);
                // 将mSnippet放入note中
                note.put(NoteColumns.SNIPPET, mSnippet);
                // 将note放入js中
                js.put(GTaskStringUtils.META_HEAD_NOTE, note);
            }

            // 返回js
            return js;
        } catch (JSONException e) {
            Log.e(TAG, e.toString());
            e.printStackTrace();
        }
        // 返回null
        return null;
    }

    public void setParentId(long id) {
        mParentId = id;
        mDiffNoteValues.put(NoteColumns.PARENT_ID, id);
    }

    public void setGtaskId(String gid) {
        mDiffNoteValues.put(NoteColumns.GTASK_ID, gid);
    }

    public void setSyncId(long syncId) {
        mDiffNoteValues.put(NoteColumns.SYNC_ID, syncId);
    }

    public void resetLocalModified() {
        mDiffNoteValues.put(NoteColumns.LOCAL_MODIFIED, 0);
    }

    public long getId() {
        return mId;
    }

    public long getParentId() {
        return mParentId;
    }

    public String getSnippet() {
        return mSnippet;
    }

    public boolean isNoteType() {
        return mType == Notes.TYPE_NOTE;
    }

    public void commit(boolean validateVersion) {
        // 如果是创建
        if (mIsCreate) {
            // 如果id为无效id且mDiffNoteValues中包含NoteColumns.ID
            if (mId == INVALID_ID && mDiffNoteValues.containsKey(NoteColumns.ID)) {
                // 从mDiffNoteValues中移除NoteColumns.ID
                mDiffNoteValues.remove(NoteColumns.ID);
            }

            // 插入数据到Notes.CONTENT_NOTE_URI，返回uri
            Uri uri = mContentResolver.insert(Notes.CONTENT_NOTE_URI, mDiffNoteValues);
            try {
                // 从uri中获取id
                mId = Long.valueOf(uri.getPathSegments().get(1));
            } catch (NumberFormatException e) {
                // 如果获取id失败，打印错误日志，抛出ActionFailureException
                Log.e(TAG, "Get note id error :" + e.toString());
                throw new ActionFailureException("create note failed");
            }
            // 如果id为0，抛出IllegalStateException
            if (mId == 0) {
                throw new IllegalStateException("Create thread id failed");
            }

            // 如果类型为Notes.TYPE_NOTE
            if (mType == Notes.TYPE_NOTE) {
                // 遍历mDataList，调用commit方法
                for (SqlData sqlData : mDataList) {
                    sqlData.commit(mId, false, -1);
                }
            }
        } else {
            // 如果id小于等于0且id不是Notes.ID_ROOT_FOLDER和Notes.ID_CALL_RECORD_FOLDER
            if (mId <= 0 && mId != Notes.ID_ROOT_FOLDER && mId != Notes.ID_CALL_RECORD_FOLDER) {
                // 打印错误日志，抛出IllegalStateException
                Log.e(TAG, "No such note");
                throw new IllegalStateException("Try to update note with invalid id");
            }
            // 如果mDiffNoteValues不为空
            if (mDiffNoteValues.size() > 0) {
                // 版本号加1
                mVersion ++;
                int result = 0;
                // 如果不验证版本号
                if (!validateVersion) {
                    // 更新数据
                    result = mContentResolver.update(Notes.CONTENT_NOTE_URI, mDiffNoteValues, "("
                            + NoteColumns.ID + "=?)", new String[] {
                        String.valueOf(mId)
                    });
                } else {
                    // 更新数据，验证版本号
                    result = mContentResolver.update(Notes.CONTENT_NOTE_URI, mDiffNoteValues, "("
                            + NoteColumns.ID + "=?) AND (" + NoteColumns.VERSION + "<=?)",
                            new String[] {
                                    String.valueOf(mId), String.valueOf(mVersion)
                            });
                }
                // 如果没有更新，打印警告日志
                if (result == 0) {
                    Log.w(TAG, "there is no update. maybe user updates note when syncing");
                }
            }

            // 如果类型为Notes.TYPE_NOTE
            if (mType == Notes.TYPE_NOTE) {
                // 遍历mDataList，调用commit方法
                for (SqlData sqlData : mDataList) {
                    sqlData.commit(mId, validateVersion, mVersion);
                }
            }
        }

        // refresh local info
        loadFromCursor(mId);
        if (mType == Notes.TYPE_NOTE)
            loadDataContent();

        mDiffNoteValues.clear();
        mIsCreate = false;
    }
}
