/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.tool;

import android.content.ContentProviderOperation;
import android.content.ContentProviderResult;
import android.content.ContentResolver;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.OperationApplicationException;
import android.database.Cursor;
import android.os.RemoteException;
import android.util.Log;

import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.CallNote;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.ui.NotesListAdapter.AppWidgetAttribute;

import java.util.ArrayList;
import java.util.HashSet;


public class DataUtils {
    // 定义日志标签
    public static final String TAG = "DataUtils";
    // 定义一个静态方法batchDeleteNotes，用于批量删除笔记
    public static boolean batchDeleteNotes(ContentResolver resolver, HashSet<Long> ids) {
        // 检查传入的ids是否为null
        if (ids == null) {
            // 如果ids为null，记录日志并返回true
            Log.d(TAG, "the ids is null");
            return true;
        }
        // 检查ids集合的大小是否为0
        if (ids.size() == 0) {
            // 如果ids集合为空，记录日志并返回true
            Log.d(TAG, "no id is in the hashset");
            return true;
        }

        // 创建一个ArrayList来存储ContentProviderOperation对象
        ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>();
        // 遍历ids集合中的每个id
        for (long id : ids) {
            // 检查当前id是否为系统文件夹根ID
            if(id == Notes.ID_ROOT_FOLDER) {
                // 如果是系统文件夹根ID，记录错误日志并跳过当前循环
                Log.e(TAG, "Don't delete system folder root");
                continue;
            }
            // 创建一个ContentProviderOperation.Builder对象，用于构建删除操作
            ContentProviderOperation.Builder builder = ContentProviderOperation
                    .newDelete(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id));
            // 将构建好的删除操作添加到操作列表中
            operationList.add(builder.build());
        }
        try {
            // 使用ContentResolver的applyBatch方法批量执行操作列表中的操作
            ContentProviderResult[] results = resolver.applyBatch(Notes.AUTHORITY, operationList);
            // 检查操作结果是否为空或长度为0，或者第一个结果为null
            if (results == null || results.length == 0 || results[0] == null) {
                // 如果操作失败，记录日志并返回false
                Log.d(TAG, "delete notes failed, ids:" + ids.toString());
                return false;
            }
            // 如果操作成功，返回true
            return true;
        } catch (RemoteException e) {
            // 捕获RemoteException异常，记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        } catch (OperationApplicationException e) {
            // 捕获OperationApplicationException异常，记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        }
        // 如果发生异常，返回false
        return false;
    }

    // 定义一个静态方法，用于将指定的笔记移动到目标文件夹
    public static void moveNoteToFoler(ContentResolver resolver, long id, long srcFolderId, long desFolderId) {
        // 创建一个ContentValues对象，用于存储要更新的数据
        ContentValues values = new ContentValues();
        // 将目标文件夹ID放入ContentValues对象中，键为NoteColumns.PARENT_ID
        values.put(NoteColumns.PARENT_ID, desFolderId);
        // 将源文件夹ID放入ContentValues对象中，键为NoteColumns.ORIGIN_PARENT_ID
        values.put(NoteColumns.ORIGIN_PARENT_ID, srcFolderId);
        // 将本地修改标志设置为1，表示该笔记已被本地修改，键为NoteColumns.LOCAL_MODIFIED
        values.put(NoteColumns.LOCAL_MODIFIED, 1);
        // 使用ContentResolver的update方法更新数据库中的笔记
        // ContentUris.withAppendedId方法将笔记ID附加到CONTENT_NOTE_URI上，形成具体的笔记URI
        // values为要更新的数据，null和null为更新条件，这里表示更新所有匹配的记录
        resolver.update(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id), values, null, null);
    }

    // 定义一个静态方法，用于将一组笔记批量移动到指定文件夹
    public static boolean batchMoveToFolder(ContentResolver resolver, HashSet<Long> ids,
            long folderId) {
        // 检查传入的ids集合是否为空
        if (ids == null) {
            // 如果为空，记录日志并返回true（表示没有需要移动的笔记）
            Log.d(TAG, "the ids is null");
            return true;
        }

        // 创建一个用于存储ContentProvider操作的列表
        ArrayList<ContentProviderOperation> operationList = new ArrayList<ContentProviderOperation>();
        // 遍历ids集合中的每个id
        for (long id : ids) {
            // 创建一个ContentProviderOperation的构建器，用于更新指定id的笔记
            ContentProviderOperation.Builder builder = ContentProviderOperation
                    .newUpdate(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, id));
            // 设置笔记的新父文件夹ID
            builder.withValue(NoteColumns.PARENT_ID, folderId);
            // 设置笔记的本地修改标志为1，表示已修改
            builder.withValue(NoteColumns.LOCAL_MODIFIED, 1);
            // 将构建的ContentProviderOperation添加到操作列表中
            operationList.add(builder.build());
        }

        // 尝试执行批量操作
        try {
            // 使用ContentResolver的applyBatch方法执行操作列表中的所有操作
            ContentProviderResult[] results = resolver.applyBatch(Notes.AUTHORITY, operationList);
            // 检查操作结果是否为空或无效
            if (results == null || results.length == 0 || results[0] == null) {
                // 如果操作失败，记录日志并返回false
                Log.d(TAG, "delete notes failed, ids:" + ids.toString());
                return false;
            }
            // 如果操作成功，返回true
            return true;
        } catch (RemoteException e) {
            // 捕获RemoteException异常，记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        } catch (OperationApplicationException e) {
            // 捕获OperationApplicationException异常，记录错误日志
            Log.e(TAG, String.format("%s: %s", e.toString(), e.getMessage()));
        }
        // 如果发生异常，返回false表示操作失败
        return false;
    }

    /**
     * Get the all folder count except system folders {@link Notes#TYPE_SYSTEM}}
     */
    // 定义一个静态方法，用于获取用户文件夹的数量
    public static int getUserFolderCount(ContentResolver resolver) {
        Cursor cursor =resolver.query(Notes.CONTENT_NOTE_URI,
                new String[] { "COUNT(*)" },
                NoteColumns.TYPE + "=? AND " + NoteColumns.PARENT_ID + "<>?",
                new String[] { String.valueOf(Notes.TYPE_FOLDER), String.valueOf(Notes.ID_TRASH_FOLER)},
                null);

        int count = 0;
        if(cursor != null) {
            if(cursor.moveToFirst()) {
                try {
                    count = cursor.getInt(0);
                } catch (IndexOutOfBoundsException e) {
                    Log.e(TAG, "get folder count failed:" + e.toString());
                } finally {
                    cursor.close();
                }
            }
        }
        return count;
    }

    // 定义一个静态方法，用于检查指定类型的笔记是否在笔记数据库中可见
    public static boolean visibleInNoteDatabase(ContentResolver resolver, long noteId, int type) {
        Cursor cursor = resolver.query(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId),
                null,
                NoteColumns.TYPE + "=? AND " + NoteColumns.PARENT_ID + "<>" + Notes.ID_TRASH_FOLER,
                new String [] {String.valueOf(type)},
                null);

        boolean exist = false;
        if (cursor != null) {
            if (cursor.getCount() > 0) {
                exist = true;
            }
            cursor.close();
        }
        return exist;
    }

    // 定义一个静态方法，用于检查指定ID的笔记是否存在于数据库中
    public static boolean existInNoteDatabase(ContentResolver resolver, long noteId) {
        // 使用ContentResolver查询数据库，获取指定ID的笔记
        // ContentUris.withAppendedId方法将noteId附加到CONTENT_NOTE_URI上，形成完整的URI
        Cursor cursor = resolver.query(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, noteId),
                null, null, null, null);

        // 初始化一个布尔变量exist，用于存储查询结果
        boolean exist = false;
        // 检查查询结果是否为空
        if (cursor != null) {
            // 如果查询结果不为空，检查结果集的大小
            if (cursor.getCount() > 0) {
                // 如果结果集大小大于0，说明存在指定ID的笔记，将exist设置为true
                exist = true;
            }
            // 关闭Cursor，释放资源
            cursor.close();
        }
        // 返回查询结果
        return exist;
    }

    // 定义一个静态方法，用于检查指定ID的数据是否存在于数据数据库中
    public static boolean existInDataDatabase(ContentResolver resolver, long dataId) {
        // 使用ContentResolver查询数据，构建查询URI，将dataId附加到Notes.CONTENT_DATA_URI后面
        Cursor cursor = resolver.query(ContentUris.withAppendedId(Notes.CONTENT_DATA_URI, dataId),
                null, null, null, null);

        // 初始化一个布尔变量exist，用于存储查询结果
        boolean exist = false;
        // 检查查询结果是否为空
        if (cursor != null) {
            // 如果查询结果不为空，检查查询结果的数量是否大于0
            if (cursor.getCount() > 0) {
                // 如果查询结果数量大于0，说明数据存在，将exist设置为true
                exist = true;
            }
            // 关闭Cursor，释放资源
            cursor.close();
        }
        // 返回查询结果
        return exist;
    }

    // 定义一个静态方法checkVisibleFolderName，用于检查指定名称的文件夹是否存在于非回收站中
    public static boolean checkVisibleFolderName(ContentResolver resolver, String name) {
        Cursor cursor = resolver.query(Notes.CONTENT_NOTE_URI, null,
                NoteColumns.TYPE + "=" + Notes.TYPE_FOLDER +
                " AND " + NoteColumns.PARENT_ID + "<>" + Notes.ID_TRASH_FOLER +
                " AND " + NoteColumns.SNIPPET + "=?",
                new String[] { name }, null);
        boolean exist = false;
        if(cursor != null) {
            if(cursor.getCount() > 0) {
                exist = true;
            }
            cursor.close();
        }
        return exist;
    }

    // 定义一个静态方法，用于获取指定文件夹下的便签小部件属性集合
    public static HashSet<AppWidgetAttribute> getFolderNoteWidget(ContentResolver resolver, long folderId) {
        // 使用ContentResolver查询便签内容提供者中的数据
        // Notes.CONTENT_NOTE_URI 是便签内容的URI
        // 查询的列包括 NoteColumns.WIDGET_ID 和 NoteColumns.WIDGET_TYPE
        // 查询条件是 NoteColumns.PARENT_ID 等于指定的文件夹ID
        // 查询结果按默认顺序返回
        Cursor c = resolver.query(Notes.CONTENT_NOTE_URI,
                new String[] { NoteColumns.WIDGET_ID, NoteColumns.WIDGET_TYPE },
                NoteColumns.PARENT_ID + "=?",
                new String[] { String.valueOf(folderId) },
                null);

        // 初始化一个HashSet用于存储AppWidgetAttribute对象
        HashSet<AppWidgetAttribute> set = null;
        // 检查查询结果是否为空
        if (c != null) {
            // 如果查询结果不为空，且存在数据
            if (c.moveToFirst()) {
                // 初始化HashSet
                set = new HashSet<AppWidgetAttribute>();
                // 遍历查询结果
                do {
                    try {
                        // 创建一个新的AppWidgetAttribute对象
                        AppWidgetAttribute widget = new AppWidgetAttribute();
                        // 从查询结果中获取WIDGET_ID列的值，并赋给widget对象的widgetId属性
                        widget.widgetId = c.getInt(0);
                        // 从查询结果中获取WIDGET_TYPE列的值，并赋给widget对象的widgetType属性
                        widget.widgetType = c.getInt(1);
                        // 将widget对象添加到HashSet中
                        set.add(widget);
                    } catch (IndexOutOfBoundsException e) {
                        // 捕获并记录数组越界异常
                        Log.e(TAG, e.toString());
                    }
                } while (c.moveToNext()); // 移动到下一行数据
            }
            // 关闭Cursor对象，释放资源
            c.close();
        }
        // 返回存储AppWidgetAttribute对象的HashSet
        return set;
    }

    // 定义一个静态方法，用于通过笔记ID获取电话号码
    public static String getCallNumberByNoteId(ContentResolver resolver, long noteId) {
        // 使用ContentResolver查询笔记内容URI，获取电话号码
        Cursor cursor = resolver.query(Notes.CONTENT_DATA_URI,
                // 查询结果只需要电话号码这一列
                new String [] { CallNote.PHONE_NUMBER },
                // 查询条件：笔记ID等于指定值且MIME类型为电话记录类型
                CallNote.NOTE_ID + "=? AND " + CallNote.MIME_TYPE + "=?",
                // 查询参数：笔记ID和MIME类型的具体值
                new String [] { String.valueOf(noteId), CallNote.CONTENT_ITEM_TYPE },
                // 排序参数：此处不需要排序，所以为null
                null);

        // 检查查询结果是否不为空且能够移动到第一行
        if (cursor != null && cursor.moveToFirst()) {
            try {
                // 尝试从查询结果中获取电话号码列的值
                return cursor.getString(0);
            } catch (IndexOutOfBoundsException e) {
                // 如果发生索引越界异常，记录错误日志
                Log.e(TAG, "Get call number fails " + e.toString());
            } finally {
                // 确保无论是否发生异常，查询结果集都会被关闭
                cursor.close();
            }
        }
        // 如果查询结果为空或无法获取电话号码，返回空字符串
        return "";
    }

    // 定义一个静态方法，用于通过电话号码和通话日期获取笔记ID
    public static long getNoteIdByPhoneNumberAndCallDate(ContentResolver resolver, String phoneNumber, long callDate) {
        // 使用ContentResolver查询数据库，获取符合条件的Cursor对象
        Cursor cursor = resolver.query(Notes.CONTENT_DATA_URI,
                // 查询结果只需要CallNote.NOTE_ID这一列
                new String [] { CallNote.NOTE_ID },
                // 查询条件：通话日期等于callDate，MIME类型等于CallNote.CONTENT_ITEM_TYPE，并且电话号码匹配
                CallNote.CALL_DATE + "=? AND " + CallNote.MIME_TYPE + "=? AND PHONE_NUMBERS_EQUAL("
                + CallNote.PHONE_NUMBER + ",?)",
                // 查询条件的参数：通话日期、MIME类型、电话号码
                new String [] { String.valueOf(callDate), CallNote.CONTENT_ITEM_TYPE, phoneNumber },
                // 排序方式：此处为null，表示不排序
                null);

        // 检查Cursor对象是否为空
        if (cursor != null) {
            // 尝试将Cursor移动到第一行
            if (cursor.moveToFirst()) {
                try {
                    // 返回Cursor中第一列的值，即NOTE_ID
                    return cursor.getLong(0);
                } catch (IndexOutOfBoundsException e) {
                    // 如果发生索引越界异常，记录错误日志
                    Log.e(TAG, "Get call note id fails " + e.toString());
                }
            }
            // 关闭Cursor对象，释放资源
            cursor.close();
        }
        // 如果没有找到符合条件的记录，返回0
        return 0;
    }

    // 定义一个静态方法，用于根据笔记ID获取笔记摘要
    public static String getSnippetById(ContentResolver resolver, long noteId) {
        Cursor cursor = resolver.query(Notes.CONTENT_NOTE_URI,
                new String [] { NoteColumns.SNIPPET },
                NoteColumns.ID + "=?",
                new String [] { String.valueOf(noteId)},
                null);

        if (cursor != null) {
            String snippet = "";
            if (cursor.moveToFirst()) {
                snippet = cursor.getString(0);
            }
            cursor.close();
            return snippet;
        }
        throw new IllegalArgumentException("Note is not found with id: " + noteId);
    }

    // 定义一个公共的静态方法，用于获取格式化后的代码片段
    public static String getFormattedSnippet(String snippet) {
        // 检查传入的代码片段是否为null
        if (snippet != null) {
            // 去除代码片段两端的空白字符
            snippet = snippet.trim();
            // 查找代码片段中的第一个换行符的位置
            int index = snippet.indexOf('\n');
            // 如果找到了换行符
            if (index != -1) {
                // 截取从开始到第一个换行符之前的子字符串
                snippet = snippet.substring(0, index);
            }
        }
        // 返回处理后的代码片段
        return snippet;
    }
}
