/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.content.DialogInterface.OnDismissListener;
import android.content.Intent;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.media.RingtoneManager;
import android.net.Uri;
import android.os.Bundle;
import android.os.PowerManager;
import android.provider.Settings;
import android.view.Window;
import android.view.WindowManager;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.tool.DataUtils;

import java.io.IOException;


public class AlarmAlertActivity extends Activity implements OnClickListener, OnDismissListener {
    // 定义笔记ID和笔记摘要
    private long mNoteId;
    private String mSnippet;
    // 定义笔记摘要的最大预览长度
    private static final int SNIPPET_PREW_MAX_LEN = 60;
    // 定义MediaPlayer对象用于播放闹钟声音
    MediaPlayer mPlayer;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 去掉标题栏
        requestWindowFeature(Window.FEATURE_NO_TITLE);

        // 获取当前窗口对象
        final Window win = getWindow();
        // 设置窗口标志，使应用在锁屏时也能显示
        win.addFlags(WindowManager.LayoutParams.FLAG_SHOW_WHEN_LOCKED);

        // 如果屏幕未亮，则设置窗口标志以保持屏幕常亮并点亮屏幕
        if (!isScreenOn()) {
            win.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_TURN_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_ALLOW_LOCK_WHILE_SCREEN_ON
                    | WindowManager.LayoutParams.FLAG_LAYOUT_INSET_DECOR);
        }

        // 获取启动当前Activity的Intent
        Intent intent = getIntent();

        try {
            // 从Intent中获取笔记ID
            mNoteId = Long.valueOf(intent.getData().getPathSegments().get(1));
            // 根据笔记ID获取笔记摘要
            mSnippet = DataUtils.getSnippetById(this.getContentResolver(), mNoteId);
            // 如果笔记摘要长度超过最大预览长度，则截取并添加省略号
            mSnippet = mSnippet.length() > SNIPPET_PREW_MAX_LEN ? mSnippet.substring(0,
                    SNIPPET_PREW_MAX_LEN) + getResources().getString(R.string.notelist_string_info)
                    : mSnippet;
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
            return;
        }

        // 初始化MediaPlayer对象
        mPlayer = new MediaPlayer();
        // 如果笔记在数据库中可见，则显示操作对话框并播放闹钟声音
        if (DataUtils.visibleInNoteDatabase(getContentResolver(), mNoteId, Notes.TYPE_NOTE)) {
            showActionDialog();
            playAlarmSound();
        } else {
            // 否则结束当前Activity
            finish();
        }
    }

    // 判断屏幕是否亮起
    private boolean isScreenOn() {
        PowerManager pm = (PowerManager) getSystemService(Context.POWER_SERVICE);
        return pm.isScreenOn();
    }

    // 播放闹钟声音
    private void playAlarmSound() {
        // 获取默认闹钟铃声的Uri
        Uri url = RingtoneManager.getActualDefaultRingtoneUri(this, RingtoneManager.TYPE_ALARM);

        // 获取当前静音模式下的音频流设置
        int silentModeStreams = Settings.System.getInt(getContentResolver(),
                Settings.System.MODE_RINGER_STREAMS_AFFECTED, 0);

        // 如果闹钟音频流在静音模式下受影响，则设置MediaPlayer的音频流类型为静音模式下的音频流类型
        if ((silentModeStreams & (1 << AudioManager.STREAM_ALARM)) != 0) {
            mPlayer.setAudioStreamType(silentModeStreams);
        } else {
            mPlayer.setAudioStreamType(AudioManager.STREAM_ALARM);
        }
        try {
            // 设置MediaPlayer的数据源
            mPlayer.setDataSource(this, url);
            // 准备MediaPlayer
            mPlayer.prepare();
            // 设置MediaPlayer循环播放
            mPlayer.setLooping(true);
            // 开始播放
            mPlayer.start();
        } catch (IllegalArgumentException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (SecurityException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IllegalStateException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    // 定义一个私有方法showActionDialog，用于显示一个操作对话框
    private void showActionDialog() {
        // 创建一个AlertDialog.Builder对象，用于构建对话框
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        // 设置对话框的标题，使用资源文件中的app_name字符串
        dialog.setTitle(R.string.app_name);
        // 设置对话框的消息内容，使用成员变量mSnippet的值
        dialog.setMessage(mSnippet);
        // 设置对话框的确定按钮，按钮文本使用资源文件中的notealert_ok字符串，点击事件监听器为当前对象
        dialog.setPositiveButton(R.string.notealert_ok, this);
        // 检查屏幕是否处于开启状态
        if (isScreenOn()) {
            // 如果屏幕开启，则设置对话框的取消按钮，按钮文本使用资源文件中的notealert_enter字符串，点击事件监听器为当前对象
            dialog.setNegativeButton(R.string.notealert_enter, this);
        }
        // 显示对话框，并设置对话框消失时的监听器为当前对象
        dialog.show().setOnDismissListener(this);
    }

    // 定义一个方法来处理对话框点击事件
    public void onClick(DialogInterface dialog, int which) {
        // 使用switch语句根据点击的按钮类型进行不同的处理
        switch (which) {
            // 如果点击的是否定按钮（例如“取消”或“否”）
            case DialogInterface.BUTTON_NEGATIVE:
                // 创建一个新的Intent对象，用于启动NoteEditActivity
                Intent intent = new Intent(this, NoteEditActivity.class);
                // 设置Intent的动作类型为查看（ACTION_VIEW）
                intent.setAction(Intent.ACTION_VIEW);
                // 向Intent中添加一个额外的数据，即笔记的ID
                intent.putExtra(Intent.EXTRA_UID, mNoteId);
                // 启动NoteEditActivity，传递Intent对象
                startActivity(intent);
                break;
            // 默认情况下，不进行任何操作
            default:
                break;
        }
    }

    // 定义一个方法，当对话框被取消或关闭时调用
    public void onDismiss(DialogInterface dialog) {
        // 调用stopAlarmSound方法，停止报警声音
        stopAlarmSound();
        // 调用finish方法，结束当前Activity
        finish();
    }

    // 停止报警声音的方法
    private void stopAlarmSound() {
        // 检查mPlayer对象是否不为空，以避免空指针异常
        if (mPlayer != null) {
            // 调用mPlayer的stop方法停止播放声音
            mPlayer.stop();
            // 调用mPlayer的release方法释放资源
            mPlayer.release();
            // 将mPlayer对象设置为null，表示不再使用
            mPlayer = null;
        }
    }
}
