/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import java.util.Calendar;

import net.micode.notes.R;
import net.micode.notes.ui.DateTimePicker;
import net.micode.notes.ui.DateTimePicker.OnDateTimeChangedListener;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.text.format.DateFormat;
import android.text.format.DateUtils;

public class DateTimePickerDialog extends AlertDialog implements OnClickListener {

    // 定义一个日历对象，用于存储日期和时间
    private Calendar mDate = Calendar.getInstance();
    // 是否使用24小时制
    private boolean mIs24HourView;
    // 日期时间设置监听器
    private OnDateTimeSetListener mOnDateTimeSetListener;
    // 日期时间选择器
    private DateTimePicker mDateTimePicker;

    // 定义一个接口，用于监听日期时间的设置
    public interface OnDateTimeSetListener {
        void OnDateTimeSet(AlertDialog dialog, long date);
    }

    // 构造函数，初始化对话框
    public DateTimePickerDialog(Context context, long date) {
        super(context);
        // 创建日期时间选择器
        mDateTimePicker = new DateTimePicker(context);
        // 设置对话框的视图为日期时间选择器
        setView(mDateTimePicker);
        // 设置日期时间选择器的监听器
        mDateTimePicker.setOnDateTimeChangedListener(new OnDateTimeChangedListener() {
            public void onDateTimeChanged(DateTimePicker view, int year, int month,
                    int dayOfMonth, int hourOfDay, int minute) {
                // 更新日历对象的日期和时间
                mDate.set(Calendar.YEAR, year);
                mDate.set(Calendar.MONTH, month);
                mDate.set(Calendar.DAY_OF_MONTH, dayOfMonth);
                mDate.set(Calendar.HOUR_OF_DAY, hourOfDay);
                mDate.set(Calendar.MINUTE, minute);
                // 更新对话框标题
                updateTitle(mDate.getTimeInMillis());
            }
        });
        // 设置日历对象的时间
        mDate.setTimeInMillis(date);
        mDate.set(Calendar.SECOND, 0);
        mDateTimePicker.setCurrentDate(mDate.getTimeInMillis());
        setButton(context.getString(R.string.datetime_dialog_ok), this);
        setButton2(context.getString(R.string.datetime_dialog_cancel), (OnClickListener)null);
        set24HourView(DateFormat.is24HourFormat(this.getContext()));
        updateTitle(mDate.getTimeInMillis());
    }

    // 定义一个公共方法，用于设置是否使用24小时制视图
    public void set24HourView(boolean is24HourView) {
        // 将传入的布尔值参数is24HourView赋值给成员变量mIs24HourView
        mIs24HourView = is24HourView;
    }

    // 定义一个方法用于设置日期时间设置监听器
    public void setOnDateTimeSetListener(OnDateTimeSetListener callBack) {
        // 将传入的监听器对象赋值给成员变量mOnDateTimeSetListener
        mOnDateTimeSetListener = callBack;
    }

    // 定义一个私有方法updateTitle，用于更新标题，接收一个长整型参数date表示日期时间
    private void updateTitle(long date) {
        // 定义一个整型变量flag，用于存储日期时间的格式标志
        int flag =
            // 使用位或运算符将FORMAT_SHOW_YEAR标志添加到flag中，表示显示年份
            DateUtils.FORMAT_SHOW_YEAR |
            // 使用位或运算符将FORMAT_SHOW_DATE标志添加到flag中，表示显示日期
            DateUtils.FORMAT_SHOW_DATE |
            // 使用位或运算符将FORMAT_SHOW_TIME标志添加到flag中，表示显示时间
            DateUtils.FORMAT_SHOW_TIME;
        // 使用位或运算符将FORMAT_24HOUR标志添加到flag中
        // 如果mIs24HourView为true，则添加FORMAT_24HOUR标志，表示使用24小时制
        // 如果mIs24HourView为false，则也添加FORMAT_24HOUR标志，这里可能是一个逻辑错误，应该根据mIs24HourView的值决定是否添加该标志
        flag |= mIs24HourView ? DateUtils.FORMAT_24HOUR : DateUtils.FORMAT_24HOUR;
        // 调用DateUtils的formatDateTime方法，将日期时间格式化为字符串
        // this.getContext()获取当前上下文
        // date为要格式化的日期时间
        // flag为格式标志
        setTitle(DateUtils.formatDateTime(this.getContext(), date, flag));
    }

    // 定义一个方法，当对话框被点击时触发
    public void onClick(DialogInterface arg0, int arg1) {
        // 检查mOnDateTimeSetListener是否不为空，确保有监听器注册
        if (mOnDateTimeSetListener != null) {
            // 调用mOnDateTimeSetListener的OnDateTimeSet方法，传递当前对象和日期的毫秒值
            mOnDateTimeSetListener.OnDateTimeSet(this, mDate.getTimeInMillis());
        }
    }

}