/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.app.Activity;
import android.app.AlarmManager;
import android.app.AlertDialog;
import android.app.PendingIntent;
import android.app.SearchManager;
import android.appwidget.AppWidgetManager;
import android.content.ContentUris;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Paint;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.Spannable;
import android.text.SpannableString;
import android.text.TextUtils;
import android.text.format.DateUtils;
import android.text.style.BackgroundColorSpan;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.WindowManager;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.TextNote;
import net.micode.notes.model.WorkingNote;
import net.micode.notes.model.WorkingNote.NoteSettingChangedListener;
import net.micode.notes.tool.DataUtils;
import net.micode.notes.tool.ResourceParser;
import net.micode.notes.tool.ResourceParser.TextAppearanceResources;
import net.micode.notes.ui.DateTimePickerDialog.OnDateTimeSetListener;
import net.micode.notes.ui.NoteEditText.OnTextViewChangeListener;
import net.micode.notes.widget.NoteWidgetProvider_2x;
import net.micode.notes.widget.NoteWidgetProvider_4x;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class NoteEditActivity extends Activity implements OnClickListener,
        NoteSettingChangedListener, OnTextViewChangeListener {
    // 定义一个内部类，用于保存头部视图的引用
    private class HeadViewHolder {
        public TextView tvModified; // 显示修改日期的TextView

        public ImageView ivAlertIcon; // 显示提醒图标的ImageView

        public TextView tvAlertDate; // 显示提醒日期的TextView

        public ImageView ibSetBgColor; // 设置背景颜色的ImageView
    }

    // 定义背景颜色选择按钮与背景颜色值的映射
    private static final Map<Integer, Integer> sBgSelectorBtnsMap = new HashMap<Integer, Integer>();
    static {
        sBgSelectorBtnsMap.put(R.id.iv_bg_yellow, ResourceParser.YELLOW);
        sBgSelectorBtnsMap.put(R.id.iv_bg_red, ResourceParser.RED);
        sBgSelectorBtnsMap.put(R.id.iv_bg_blue, ResourceParser.BLUE);
        sBgSelectorBtnsMap.put(R.id.iv_bg_green, ResourceParser.GREEN);
        sBgSelectorBtnsMap.put(R.id.iv_bg_white, ResourceParser.WHITE);
    }

    // 定义背景颜色值与选中状态的映射
    private static final Map<Integer, Integer> sBgSelectorSelectionMap = new HashMap<Integer, Integer>();
    static {
        sBgSelectorSelectionMap.put(ResourceParser.YELLOW, R.id.iv_bg_yellow_select);
        sBgSelectorSelectionMap.put(ResourceParser.RED, R.id.iv_bg_red_select);
        sBgSelectorSelectionMap.put(ResourceParser.BLUE, R.id.iv_bg_blue_select);
        sBgSelectorSelectionMap.put(ResourceParser.GREEN, R.id.iv_bg_green_select);
        sBgSelectorSelectionMap.put(ResourceParser.WHITE, R.id.iv_bg_white_select);
    }

    // 定义字体大小选择按钮与字体大小值的映射
    private static final Map<Integer, Integer> sFontSizeBtnsMap = new HashMap<Integer, Integer>();
    static {
        sFontSizeBtnsMap.put(R.id.ll_font_large, ResourceParser.TEXT_LARGE);
        sFontSizeBtnsMap.put(R.id.ll_font_small, ResourceParser.TEXT_SMALL);
        sFontSizeBtnsMap.put(R.id.ll_font_normal, ResourceParser.TEXT_MEDIUM);
        sFontSizeBtnsMap.put(R.id.ll_font_super, ResourceParser.TEXT_SUPER);
    }

    // 定义字体大小值与选中状态的映射
    private static final Map<Integer, Integer> sFontSelectorSelectionMap = new HashMap<Integer, Integer>();
    static {
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_LARGE, R.id.iv_large_select);
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_SMALL, R.id.iv_small_select);
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_MEDIUM, R.id.iv_medium_select);
        sFontSelectorSelectionMap.put(ResourceParser.TEXT_SUPER, R.id.iv_super_select);
    }

    private static final String TAG = "NoteEditActivity"; // 日志标签

    private HeadViewHolder mNoteHeaderHolder; // 头部视图持有者

    private View mHeadViewPanel; // 头部视图面板

    private View mNoteBgColorSelector; // 背景颜色选择器

    private View mFontSizeSelector; // 字体大小选择器

    private EditText mNoteEditor; // 笔记编辑器

    private View mNoteEditorPanel; // 笔记编辑器面板

    private WorkingNote mWorkingNote; // 当前正在编辑的笔记

    private SharedPreferences mSharedPrefs; // 共享偏好设置
    private int mFontSizeId; // 当前字体大小ID

    private static final String PREFERENCE_FONT_SIZE = "pref_font_size"; // 字体大小偏好设置键

    private static final int SHORTCUT_ICON_TITLE_MAX_LEN = 10; // 快捷方式图标标题最大长度

    public static final String TAG_CHECKED = String.valueOf('\u221A'); // 已选中标签
    public static final String TAG_UNCHECKED = String.valueOf('\u25A1'); // 未选中标签

    private LinearLayout mEditTextList; // 编辑文本列表

    private String mUserQuery; // 用户查询字符串
    private Pattern mPattern; // 正则表达式模式

    @Override
    // 重写父类的方法，用于在Activity创建时调用
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 调用父类的onCreate方法，进行Activity的基本初始化
        this.setContentView(R.layout.note_edit);

        // 设置当前Activity的布局文件为R.layout.note_edit
        if (savedInstanceState == null && !initActivityState(getIntent())) {
            // 检查savedInstanceState是否为null（即Activity是否是第一次创建）
            // 并且调用initActivityState方法初始化Activity状态，如果返回false
            finish();
            // 结束当前Activity
            return;
            // 结束方法执行，不再继续向下执行
        }
        initResources();
        // 调用initResources方法，进行资源的初始化
    }

    /**
     * Current activity may be killed when the memory is low. Once it is killed, for another time
     * user load this activity, we should restore the former state
     */
    @Override
    protected void onRestoreInstanceState(Bundle savedInstanceState) {
        // 调用父类的onRestoreInstanceState方法，确保父类的恢复逻辑得以执行
        super.onRestoreInstanceState(savedInstanceState);
        // 检查savedInstanceState是否不为空，并且包含特定的键(Intent.EXTRA_UID)
        if (savedInstanceState != null && savedInstanceState.containsKey(Intent.EXTRA_UID)) {
            // 创建一个新的Intent对象，指定动作为查看(Intent.ACTION_VIEW)
            Intent intent = new Intent(Intent.ACTION_VIEW);
            // 将savedInstanceState中存储的UID值取出，并放入Intent的额外数据中
            intent.putExtra(Intent.EXTRA_UID, savedInstanceState.getLong(Intent.EXTRA_UID));
            // 调用initActivityState方法，传入intent，尝试初始化活动状态
            // 如果初始化失败，则调用finish()方法结束当前活动
            if (!initActivityState(intent)) {
                finish();
                return;
            }
            // 使用Log.d输出调试信息，表示正在从被杀死的活动中恢复
            Log.d(TAG, "Restoring from killed activity");
        }
    }

    private boolean initActivityState(Intent intent) {
        /**
         * If the user specified the {@link Intent#ACTION_VIEW} but not provided with id,
         * then jump to the NotesListActivity
         */
        mWorkingNote = null;
        if (TextUtils.equals(Intent.ACTION_VIEW, intent.getAction())) {
            long noteId = intent.getLongExtra(Intent.EXTRA_UID, 0);
            mUserQuery = "";

            /**
             * Starting from the searched result
             */
            if (intent.hasExtra(SearchManager.EXTRA_DATA_KEY)) {
                noteId = Long.parseLong(intent.getStringExtra(SearchManager.EXTRA_DATA_KEY));
                mUserQuery = intent.getStringExtra(SearchManager.USER_QUERY);
            }

            if (!DataUtils.visibleInNoteDatabase(getContentResolver(), noteId, Notes.TYPE_NOTE)) {
                Intent jump = new Intent(this, NotesListActivity.class);
                startActivity(jump);
                showToast(R.string.error_note_not_exist);
                finish();
                return false;
            } else {
                mWorkingNote = WorkingNote.load(this, noteId);
                if (mWorkingNote == null) {
                    Log.e(TAG, "load note failed with note id" + noteId);
                    finish();
                    return false;
                }
            }
            getWindow().setSoftInputMode(
                    WindowManager.LayoutParams.SOFT_INPUT_STATE_HIDDEN
                            | WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE);
        } else if(TextUtils.equals(Intent.ACTION_INSERT_OR_EDIT, intent.getAction())) {
            // New note
            long folderId = intent.getLongExtra(Notes.INTENT_EXTRA_FOLDER_ID, 0);
            int widgetId = intent.getIntExtra(Notes.INTENT_EXTRA_WIDGET_ID,
                    AppWidgetManager.INVALID_APPWIDGET_ID);
            int widgetType = intent.getIntExtra(Notes.INTENT_EXTRA_WIDGET_TYPE,
                    Notes.TYPE_WIDGET_INVALIDE);
            int bgResId = intent.getIntExtra(Notes.INTENT_EXTRA_BACKGROUND_ID,
                    ResourceParser.getDefaultBgId(this));

            // Parse call-record note
            String phoneNumber = intent.getStringExtra(Intent.EXTRA_PHONE_NUMBER);
            long callDate = intent.getLongExtra(Notes.INTENT_EXTRA_CALL_DATE, 0);
            if (callDate != 0 && phoneNumber != null) {
                if (TextUtils.isEmpty(phoneNumber)) {
                    Log.w(TAG, "The call record number is null");
                }
                long noteId = 0;
                if ((noteId = DataUtils.getNoteIdByPhoneNumberAndCallDate(getContentResolver(),
                        phoneNumber, callDate)) > 0) {
                    mWorkingNote = WorkingNote.load(this, noteId);
                    if (mWorkingNote == null) {
                        Log.e(TAG, "load call note failed with note id" + noteId);
                        finish();
                        return false;
                    }
                } else {
                    mWorkingNote = WorkingNote.createEmptyNote(this, folderId, widgetId,
                            widgetType, bgResId);
                    mWorkingNote.convertToCallNote(phoneNumber, callDate);
                }
            } else {
                mWorkingNote = WorkingNote.createEmptyNote(this, folderId, widgetId, widgetType,
                        bgResId);
            }

            getWindow().setSoftInputMode(
                    WindowManager.LayoutParams.SOFT_INPUT_ADJUST_RESIZE
                            | WindowManager.LayoutParams.SOFT_INPUT_STATE_VISIBLE);
        } else {
            Log.e(TAG, "Intent not specified action, should not support");
            finish();
            return false;
        }
        mWorkingNote.setOnSettingStatusChangedListener(this);
        return true;
    }

    @Override
    // 重写父类的方法，表示这个方法是一个覆盖方法
    protected void onResume() {
        // 调用父类的onResume方法，确保父类的逻辑得以执行
        super.onResume();
        // 调用initNoteScreen方法，初始化笔记屏幕
        initNoteScreen();
    }

    // 初始化笔记屏幕的显示
    private void initNoteScreen() {
        // 设置笔记编辑器的文本外观，根据字体大小ID获取对应的文本外观资源
        mNoteEditor.setTextAppearance(this, TextAppearanceResources
                .getTexAppearanceResource(mFontSizeId));
        // 检查当前笔记是否为待办事项模式
        if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
            // 如果是待办事项模式，则切换到列表模式，并设置内容
            switchToListMode(mWorkingNote.getContent());
        } else {
            // 如果不是待办事项模式，则设置笔记编辑器的文本内容
            // 高亮显示用户查询结果
            mNoteEditor.setText(getHighlightQueryResult(mWorkingNote.getContent(), mUserQuery));
            // 将光标移动到文本末尾
            mNoteEditor.setSelection(mNoteEditor.getText().length());
        }
        // 遍历背景选择器的选择映射，将所有背景选择器的视图设置为不可见
        for (Integer id : sBgSelectorSelectionMap.keySet()) {
            findViewById(sBgSelectorSelectionMap.get(id)).setVisibility(View.GONE);
        }
        // 设置笔记标题栏的背景资源
        mHeadViewPanel.setBackgroundResource(mWorkingNote.getTitleBgResId());
        // 设置笔记编辑器面板的背景资源
        mNoteEditorPanel.setBackgroundResource(mWorkingNote.getBgColorResId());

        // 设置笔记头部视图的修改时间文本
        // 使用DateUtils格式化日期时间，显示日期、时间、年份
        mNoteHeaderHolder.tvModified.setText(DateUtils.formatDateTime(this,
                mWorkingNote.getModifiedDate(), DateUtils.FORMAT_SHOW_DATE
                        | DateUtils.FORMAT_NUMERIC_DATE | DateUtils.FORMAT_SHOW_TIME
                        | DateUtils.FORMAT_SHOW_YEAR));

        /**
         * TODO: Add the menu for setting alert. Currently disable it because the DateTimePicker
         * is not ready
         */
        showAlertHeader();
    }

    // 定义一个私有方法用于显示提醒头部信息
    private void showAlertHeader() {
        // 检查当前工作笔记是否有闹钟提醒
        if (mWorkingNote.hasClockAlert()) {
            // 获取当前系统时间
            long time = System.currentTimeMillis();
            // 比较当前时间与提醒时间
            if (time > mWorkingNote.getAlertDate()) {
                // 如果当前时间大于提醒时间，说明提醒已过期
                // 设置提醒日期文本为“提醒已过期”
                mNoteHeaderHolder.tvAlertDate.setText(R.string.note_alert_expired);
            } else {
                // 如果当前时间小于提醒时间，说明提醒还未到
                // 使用DateUtils工具类获取相对时间字符串
                mNoteHeaderHolder.tvAlertDate.setText(DateUtils.getRelativeTimeSpanString(
                        mWorkingNote.getAlertDate(), time, DateUtils.MINUTE_IN_MILLIS));
            }
            // 显示提醒日期文本视图
            mNoteHeaderHolder.tvAlertDate.setVisibility(View.VISIBLE);
            // 显示提醒图标视图
            mNoteHeaderHolder.ivAlertIcon.setVisibility(View.VISIBLE);
        } else {
            // 如果当前工作笔记没有闹钟提醒
            // 隐藏提醒日期文本视图
            mNoteHeaderHolder.tvAlertDate.setVisibility(View.GONE);
            // 隐藏提醒图标视图
            mNoteHeaderHolder.ivAlertIcon.setVisibility(View.GONE);
        };
    }

    @Override
    // 重写父类的方法，当Activity接收到新的Intent时调用
    protected void onNewIntent(Intent intent) {
        super.onNewIntent(intent);
        // 调用父类的onNewIntent方法，确保父类的逻辑得以执行
        initActivityState(intent);
        // 调用initActivityState方法，传入新的Intent，用于初始化或更新Activity的状态
    }

    @Override
    protected void onSaveInstanceState(Bundle outState) {
        // 调用父类的onSaveInstanceState方法，保存当前Activity的状态
        super.onSaveInstanceState(outState);
        /**
         * For new note without note id, we should firstly save it to
         * generate a id. If the editing note is not worth saving, there
         * is no id which is equivalent to create new note
         */
        if (!mWorkingNote.existInDatabase()) {
            saveNote();
        }
        outState.putLong(Intent.EXTRA_UID, mWorkingNote.getNoteId());
        Log.d(TAG, "Save working note id: " + mWorkingNote.getNoteId() + " onSaveInstanceState");
    }

    @Override
    public boolean dispatchTouchEvent(MotionEvent ev) {
        // 检查笔记背景颜色选择器的可见性，并且判断触摸事件是否发生在选择器之外
        if (mNoteBgColorSelector.getVisibility() == View.VISIBLE
                && !inRangeOfView(mNoteBgColorSelector, ev)) {
            // 如果选择器可见且触摸事件发生在选择器之外，则隐藏选择器
            mNoteBgColorSelector.setVisibility(View.GONE);
            // 返回true表示事件已处理，不再传递
            return true;
        }

        // 检查字体大小选择器的可见性，并且判断触摸事件是否发生在选择器之外
        if (mFontSizeSelector.getVisibility() == View.VISIBLE
                && !inRangeOfView(mFontSizeSelector, ev)) {
            // 如果选择器可见且触摸事件发生在选择器之外，则隐藏选择器
            mFontSizeSelector.setVisibility(View.GONE);
            // 返回true表示事件已处理，不再传递
            return true;
        }
        // 如果上述条件都不满足，则调用父类的dispatchTouchEvent方法继续处理事件
        return super.dispatchTouchEvent(ev);
    }

    // 定义一个私有方法，用于判断给定的MotionEvent是否在指定View的可见范围内
    private boolean inRangeOfView(View view, MotionEvent ev) {
        // 创建一个整型数组，用于存储View在屏幕上的坐标位置
        int []location = new int[2];
        // 获取View在屏幕上的绝对坐标位置，结果存储在location数组中
        view.getLocationOnScreen(location);
        // 从location数组中获取View的左上角x坐标
        int x = location[0];
        // 从location数组中获取View的左上角y坐标
        int y = location[1];
        if (ev.getX() < x
                || ev.getX() > (x + view.getWidth())
                || ev.getY() < y
                || ev.getY() > (y + view.getHeight())) {
                    return false;
                }
        return true;
    }

    // 初始化资源的方法
    private void initResources() {
        // 获取标题面板的视图
        mHeadViewPanel = findViewById(R.id.note_title);
        // 创建一个HeadViewHolder对象来保存标题部分的视图
        mNoteHeaderHolder = new HeadViewHolder();
        // 获取修改日期的TextView
        mNoteHeaderHolder.tvModified = (TextView) findViewById(R.id.tv_modified_date);
        // 获取提醒图标的ImageView
        mNoteHeaderHolder.ivAlertIcon = (ImageView) findViewById(R.id.iv_alert_icon);
        // 获取提醒日期的TextView
        mNoteHeaderHolder.tvAlertDate = (TextView) findViewById(R.id.tv_alert_date);
        // 获取设置背景颜色的ImageView
        mNoteHeaderHolder.ibSetBgColor = (ImageView) findViewById(R.id.btn_set_bg_color);
        // 为设置背景颜色的ImageView设置点击监听器
        mNoteHeaderHolder.ibSetBgColor.setOnClickListener(this);
        // 获取笔记编辑器的EditText
        mNoteEditor = (EditText) findViewById(R.id.note_edit_view);
        // 获取笔记编辑面板的视图
        mNoteEditorPanel = findViewById(R.id.sv_note_edit);
        // 获取背景颜色选择器的视图
        mNoteBgColorSelector = findViewById(R.id.note_bg_color_selector);
        // 遍历背景颜色选择器的按钮ID集合，并为每个按钮设置点击监听器
        for (int id : sBgSelectorBtnsMap.keySet()) {
            ImageView iv = (ImageView) findViewById(id);
            iv.setOnClickListener(this);
        }

        // 获取字体大小选择器的视图
        mFontSizeSelector = findViewById(R.id.font_size_selector);
        // 遍历字体大小选择器的按钮ID集合，并为每个按钮设置点击监听器
        for (int id : sFontSizeBtnsMap.keySet()) {
            View view = findViewById(id);
            view.setOnClickListener(this);
        };
        // 获取默认的SharedPreferences
        mSharedPrefs = PreferenceManager.getDefaultSharedPreferences(this);
        // 从SharedPreferences中获取保存的字体大小ID，如果没有则使用默认值
        mFontSizeId = mSharedPrefs.getInt(PREFERENCE_FONT_SIZE, ResourceParser.BG_DEFAULT_FONT_SIZE);
        /**
         * HACKME: Fix bug of store the resource id in shared preference.
         * The id may larger than the length of resources, in this case,
         * return the {@link ResourceParser#BG_DEFAULT_FONT_SIZE}
         */
        if(mFontSizeId >= TextAppearanceResources.getResourcesSize()) {
            mFontSizeId = ResourceParser.BG_DEFAULT_FONT_SIZE;
        }
        mEditTextList = (LinearLayout) findViewById(R.id.note_edit_list);
    }

    @Override
    // 重写父类中的onPause方法，该方法在Activity暂停时被调用
    protected void onPause() {
        super.onPause();
        // 调用父类的onPause方法，确保父类的逻辑得以执行
        if(saveNote()) {
            // 调用saveNote方法保存笔记，如果保存成功（返回true），则执行以下代码
            Log.d(TAG, "Note data was saved with length:" + mWorkingNote.getContent().length());
            // 使用Log.d方法记录调试信息，TAG为日志标签，内容为"Note data was saved with length:"加上笔记内容的长度
        }
        clearSettingState();
        // 调用clearSettingState方法清除设置状态，确保在暂停时不会保留不必要的设置
    }

    // 更新小部件的方法
    private void updateWidget() {
        // 创建一个意图，用于发送广播，更新小部件
        Intent intent = new Intent(AppWidgetManager.ACTION_APPWIDGET_UPDATE);
        // 根据当前工作笔记的小部件类型，设置对应的意图类
        if (mWorkingNote.getWidgetType() == Notes.TYPE_WIDGET_2X) {
            // 如果是小部件类型2x2，则设置意图类为NoteWidgetProvider_2x
            intent.setClass(this, NoteWidgetProvider_2x.class);
        } else if (mWorkingNote.getWidgetType() == Notes.TYPE_WIDGET_4X) {
            // 如果是小部件类型4x4，则设置意图类为NoteWidgetProvider_4x
            intent.setClass(this, NoteWidgetProvider_4x.class);
        } else {
            // 如果是小部件类型不支持，则记录错误日志并返回
            Log.e(TAG, "Unspported widget type");
            return;
        }

        // 将小部件ID作为额外数据添加到意图中
        intent.putExtra(AppWidgetManager.EXTRA_APPWIDGET_IDS, new int[] {
            mWorkingNote.getWidgetId()
        });

        // 发送广播，通知系统更新小部件
        sendBroadcast(intent);
        // 设置结果码和意图，以便调用者知道操作成功
        setResult(RESULT_OK, intent);
    }

    // 定义一个方法来处理点击事件
    public void onClick(View v) {
        // 获取点击的视图的ID
        int id = v.getId();
        // 如果点击的是设置背景颜色的按钮
        if (id == R.id.btn_set_bg_color) {
            // 显示背景颜色选择器
            mNoteBgColorSelector.setVisibility(View.VISIBLE);
            // 隐藏当前背景颜色选择器的选中状态
            findViewById(sBgSelectorSelectionMap.get(mWorkingNote.getBgColorId())).setVisibility(
                    -                    View.VISIBLE);
        } else if (sBgSelectorBtnsMap.containsKey(id)) {
            // 隐藏当前背景颜色选择器的选中状态
            findViewById(sBgSelectorSelectionMap.get(mWorkingNote.getBgColorId())).setVisibility(
                    View.GONE);
            // 设置新的背景颜色ID
            mWorkingNote.setBgColorId(sBgSelectorBtnsMap.get(id));
            // 隐藏背景颜色选择器
            mNoteBgColorSelector.setVisibility(View.GONE);
        } else if (sFontSizeBtnsMap.containsKey(id)) {
            // 隐藏当前字体大小选择器的选中状态
            findViewById(sFontSelectorSelectionMap.get(mFontSizeId)).setVisibility(View.GONE);
            // 设置新的字体大小ID
            mFontSizeId = sFontSizeBtnsMap.get(id);
            // 将新的字体大小ID保存到SharedPreferences
            mSharedPrefs.edit().putInt(PREFERENCE_FONT_SIZE, mFontSizeId).commit();
            // 显示新的字体大小选择器的选中状态
            findViewById(sFontSelectorSelectionMap.get(mFontSizeId)).setVisibility(View.VISIBLE);
            // 如果当前笔记是检查列表模式
            if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
                // 获取当前笔记的内容
                getWorkingText();
                // 切换到检查列表模式
                switchToListMode(mWorkingNote.getContent());
            } else {
                // 设置笔记编辑器的文本外观
                mNoteEditor.setTextAppearance(this,
                        TextAppearanceResources.getTexAppearanceResource(mFontSizeId));
            }
            // 隐藏字体大小选择器
            mFontSizeSelector.setVisibility(View.GONE);
        }
    }

    @Override
    // 重写父类的方法，当用户按下返回键时调用
    public void onBackPressed() {
        // 调用clearSettingState方法，检查是否需要清除设置状态
        if(clearSettingState()) {
            // 如果clearSettingState返回true，表示需要清除设置状态，直接返回，不执行后续操作
            return;
        }

        // 调用saveNote方法，保存当前笔记
        saveNote();
        // 调用父类的onBackPressed方法，执行默认的返回键操作
        super.onBackPressed();
    }

    // 定义一个私有方法clearSettingState，用于清除设置状态
    private boolean clearSettingState() {
        // 检查笔记背景颜色选择器的可见性
        if (mNoteBgColorSelector.getVisibility() == View.VISIBLE) {
            // 如果笔记背景颜色选择器是可见的，将其设置为不可见
            mNoteBgColorSelector.setVisibility(View.GONE);
            // 返回true，表示成功隐藏了笔记背景颜色选择器
            return true;
        } else if (mFontSizeSelector.getVisibility() == View.VISIBLE) {
            // 如果字体大小选择器是可见的，将其设置为不可见
            mFontSizeSelector.setVisibility(View.GONE);
            // 返回true，表示成功隐藏了字体大小选择器
            return true;
        }
        // 如果笔记背景颜色选择器和字体大小选择器都不可见，返回false
        return false;
    }

    // 定义一个方法，当背景颜色发生变化时调用
    public void onBackgroundColorChanged() {
        // 通过查找当前工作笔记的背景颜色ID，获取对应的视图，并将其设置为可见
        findViewById(sBgSelectorSelectionMap.get(mWorkingNote.getBgColorId())).setVisibility(
                View.VISIBLE);
        // 设置笔记编辑面板的背景资源为当前工作笔记的背景颜色资源ID
        mNoteEditorPanel.setBackgroundResource(mWorkingNote.getBgColorResId());
        // 设置头部视图面板的背景资源为当前工作笔记的标题背景资源ID
        mHeadViewPanel.setBackgroundResource(mWorkingNote.getTitleBgResId());
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu) {
        // 检查当前Activity是否正在 finishing（销毁中）
        if (isFinishing()) {
            // 如果Activity正在销毁，则不执行后续操作，直接返回true
            return true;
        }
        // 清除设置状态
        clearSettingState();
        // 清空菜单
        menu.clear();
        // 根据当前工作笔记的文件夹ID，决定加载哪个菜单布局
        if (mWorkingNote.getFolderId() == Notes.ID_CALL_RECORD_FOLDER) {
            // 如果是通话记录文件夹，则加载通话笔记编辑菜单
            getMenuInflater().inflate(R.menu.call_note_edit, menu);
        } else {
            // 否则加载普通笔记编辑菜单
            getMenuInflater().inflate(R.menu.note_edit, menu);
        }
        // 根据当前工作笔记的清单模式，设置菜单项的标题
        if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
            // 如果是清单模式，则将清单模式菜单项标题设置为“普通模式”
            menu.findItem(R.id.menu_list_mode).setTitle(R.string.menu_normal_mode);
        } else {
            // 否则将清单模式菜单项标题设置为“清单模式”
            menu.findItem(R.id.menu_list_mode).setTitle(R.string.menu_list_mode);
        }
        // 根据当前工作笔记是否有闹钟提醒，设置菜单项的可见性
        if (mWorkingNote.hasClockAlert()) {
            // 如果有闹钟提醒，则隐藏“添加提醒”菜单项
            menu.findItem(R.id.menu_alert).setVisible(false);
        } else {
            // 如果没有闹钟提醒，则隐藏“删除提醒”菜单项
            menu.findItem(R.id.menu_delete_remind).setVisible(false);
        }
        // 返回true表示菜单已准备好
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // 根据菜单项的ID进行不同的操作
        switch (item.getItemId()) {
            case R.id.menu_new_note:
                // 创建新笔记
                createNewNote();
                break;
            case R.id.menu_delete:
                // 删除当前笔记
                AlertDialog.Builder builder = new AlertDialog.Builder(this);
                // 设置对话框标题
                builder.setTitle(getString(R.string.alert_title_delete));
                // 设置对话框图标
                builder.setIcon(android.R.drawable.ic_dialog_alert);
                // 设置对话框消息
                builder.setMessage(getString(R.string.alert_message_delete_note));
                // 设置对话框的确定按钮及其点击事件
                builder.setPositiveButton(android.R.string.ok,
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                // 删除当前笔记
                                deleteCurrentNote();
                                // 结束当前Activity
                                finish();
                            }
                        });
                // 设置对话框的取消按钮
                builder.setNegativeButton(android.R.string.cancel, null);
                // 显示对话框
                builder.show();
                break;
            case R.id.menu_font_size:
                // 显示字体大小选择器
                mFontSizeSelector.setVisibility(View.VISIBLE);
                // 显示当前字体大小选项
                findViewById(sFontSelectorSelectionMap.get(mFontSizeId)).setVisibility(View.VISIBLE);
                break;
            case R.id.menu_list_mode:
                // 切换笔记的列表模式
                mWorkingNote.setCheckListMode(mWorkingNote.getCheckListMode() == 0 ?
                        TextNote.MODE_CHECK_LIST : 0);
                break;
            case R.id.menu_share:
                // 获取当前笔记的内容
                getWorkingText();
                // 分享笔记内容
                sendTo(this, mWorkingNote.getContent());
                break;
            case R.id.menu_send_to_desktop:
                // 将笔记发送到桌面
                sendToDesktop();
                break;
            case R.id.menu_alert:
                // 设置提醒
                setReminder();
                break;
            case R.id.menu_delete_remind:
                // 删除提醒
                mWorkingNote.setAlertDate(0, false);
                break;
            default:
                // 默认操作
                break;
        }
        // 返回true表示事件已处理
        return true;
    }

    // 定义一个私有方法setReminder，用于设置提醒
    private void setReminder() {
        // 创建一个DateTimePickerDialog对象d，传入当前上下文和当前系统时间
        DateTimePickerDialog d = new DateTimePickerDialog(this, System.currentTimeMillis());
        // 为DateTimePickerDialog对象设置一个时间设置监听器
        d.setOnDateTimeSetListener(new OnDateTimeSetListener() {
            // 当用户设置时间时，会调用此方法
            public void OnDateTimeSet(AlertDialog dialog, long date) {
                // 将设置的时间赋值给工作笔记的提醒日期，并标记为已设置
                mWorkingNote.setAlertDate(date	, true);
            }
        });
        // 显示DateTimePickerDialog对话框
        d.show();
    }

    /**
     * Share note to apps that support {@link Intent#ACTION_SEND} action
     * and {@text/plain} type
     */
    // 定义一个私有方法sendTo，用于发送信息
    private void sendTo(Context context, String info) {
        // 创建一个Intent对象，指定动作类型为ACTION_SEND，表示发送
        Intent intent = new Intent(Intent.ACTION_SEND);
        // 将要发送的信息添加到Intent中，使用EXTRA_TEXT键，info为要发送的文本内容
        intent.putExtra(Intent.EXTRA_TEXT, info);
        // 设置Intent的数据类型为"text/plain"，表示发送的是纯文本
        intent.setType("text/plain");
        // 使用传入的Context对象启动Activity，将Intent传递给系统，系统会根据Intent的类型选择合适的Activity来处理发送操作
        context.startActivity(intent);
    }

    private void createNewNote() {
        // Firstly, save current editing notes
        saveNote();

        // For safety, start a new NoteEditActivity
        finish();
        Intent intent = new Intent(this, NoteEditActivity.class);
        intent.setAction(Intent.ACTION_INSERT_OR_EDIT);
        intent.putExtra(Notes.INTENT_EXTRA_FOLDER_ID, mWorkingNote.getFolderId());
        startActivity(intent);
    }

    // 删除当前笔记的方法
    private void deleteCurrentNote() {
        // 检查当前笔记是否存在于数据库中
        if (mWorkingNote.existInDatabase()) {
            // 创建一个HashSet用于存储要删除的笔记ID
            HashSet<Long> ids = new HashSet<Long>();
            // 获取当前笔记的ID
            long id = mWorkingNote.getNoteId();
            // 检查笔记ID是否为根文件夹ID，如果不是则添加到HashSet中
            if (id != Notes.ID_ROOT_FOLDER) {
                ids.add(id);
            } else {
                // 如果笔记ID为根文件夹ID，则记录错误日志
                Log.d(TAG, "Wrong note id, should not happen");
            }
            // 检查当前是否处于同步模式
            if (!isSyncMode()) {
                // 如果不是同步模式，则批量删除笔记
                if (!DataUtils.batchDeleteNotes(getContentResolver(), ids)) {
                    // 如果删除失败，则记录错误日志
                    Log.e(TAG, "Delete Note error");
                }
            } else {
                // 如果是同步模式，则将笔记移动到垃圾文件夹
                if (!DataUtils.batchMoveToFolder(getContentResolver(), ids, Notes.ID_TRASH_FOLER)) {
                    // 如果移动失败，则记录错误日志
                    Log.e(TAG, "Move notes to trash folder error, should not happens");
                }
            }
        }
        // 标记当前笔记为已删除
        mWorkingNote.markDeleted(true);
    }

    // 定义一个私有方法isSyncMode，用于判断当前是否处于同步模式
    private boolean isSyncMode() {
        return NotesPreferenceActivity.getSyncAccountName(this).trim().length() > 0;
    }

    public void onClockAlertChanged(long date, boolean set) {
        /**
         * User could set clock to an unsaved note, so before setting the
         * alert clock, we should save the note first
         */
        if (!mWorkingNote.existInDatabase()) {
            saveNote();
        }
        if (mWorkingNote.getNoteId() > 0) {
            Intent intent = new Intent(this, AlarmReceiver.class);
            intent.setData(ContentUris.withAppendedId(Notes.CONTENT_NOTE_URI, mWorkingNote.getNoteId()));
            PendingIntent pendingIntent = PendingIntent.getBroadcast(this, 0, intent, 0);
            AlarmManager alarmManager = ((AlarmManager) getSystemService(ALARM_SERVICE));
            showAlertHeader();
            if(!set) {
                alarmManager.cancel(pendingIntent);
            } else {
                alarmManager.set(AlarmManager.RTC_WAKEUP, date, pendingIntent);
            }
        } else {
            /**
             * There is the condition that user has input nothing (the note is
             * not worthy saving), we have no note id, remind the user that he
             * should input something
             */
            Log.e(TAG, "Clock alert setting error");
            showToast(R.string.error_note_empty_for_clock);
        }
    }

    // 定义一个公共方法，当小部件发生变化时调用
    public void onWidgetChanged() {
        // 调用updateWidget方法来更新小部件
        updateWidget();
    }

    // 定义一个方法，用于处理编辑文本框的删除操作
    public void onEditTextDelete(int index, String text) {
        // 获取当前编辑文本框列表中的子视图数量
        int childCount = mEditTextList.getChildCount();
        // 如果只有一个编辑文本框，则直接返回，不做任何操作
        if (childCount == 1) {
            return;
        }

        // 从被删除的编辑文本框的下一个开始，遍历所有后续的编辑文本框
        for (int i = index + 1; i < childCount; i++) {
            // 获取当前编辑文本框的子视图，并设置其索引为当前索引减1
            ((NoteEditText) mEditTextList.getChildAt(i).findViewById(R.id.et_edit_text))
                    .setIndex(i - 1);
        }

        // 从列表中移除被删除的编辑文本框
        mEditTextList.removeViewAt(index);
        NoteEditText edit = null;
        // 如果被删除的编辑文本框是第一个，则获取新的第一个编辑文本框
        if(index == 0) {
            edit = (NoteEditText) mEditTextList.getChildAt(0).findViewById(
                    R.id.et_edit_text);
        } else {
            // 否则获取被删除的编辑文本框的前一个编辑文本框
            edit = (NoteEditText) mEditTextList.getChildAt(index - 1).findViewById(
                    R.id.et_edit_text);
        }
        // 获取当前编辑文本框的文本长度
        int length = edit.length();
        // 将被删除的编辑文本框的文本追加到当前编辑文本框
        edit.append(text);
        // 请求焦点
        edit.requestFocus();
        // 设置光标位置为文本末尾
        edit.setSelection(length);
    }

    public void onEditTextEnter(int index, String text) {
        /**
         * Should not happen, check for debug
         */
        if(index > mEditTextList.getChildCount()) {
            Log.e(TAG, "Index out of mEditTextList boundrary, should not happen");
        }

        View view = getListItem(text, index);
        mEditTextList.addView(view, index);
        NoteEditText edit = (NoteEditText) view.findViewById(R.id.et_edit_text);
        edit.requestFocus();
        edit.setSelection(0);
        for (int i = index + 1; i < mEditTextList.getChildCount(); i++) {
            ((NoteEditText) mEditTextList.getChildAt(i).findViewById(R.id.et_edit_text))
                    .setIndex(i);
        }
    }

    // 定义一个私有方法，用于将文本转换为列表模式
    private void switchToListMode(String text) {
        // 清除mEditTextList中的所有视图
        mEditTextList.removeAllViews();
        // 将输入的文本按换行符分割成字符串数组
        String[] items = text.split("\n");
        // 初始化索引变量
        int index = 0;
        // 遍历分割后的字符串数组
        for (String item : items) {
            // 检查当前字符串是否为空
            if(!TextUtils.isEmpty(item)) {
                // 如果不为空，将当前字符串作为列表项添加到mEditTextList中
                mEditTextList.addView(getListItem(item, index));
                // 索引加1
                index++;
            }
        }
        // 添加一个空列表项到mEditTextList中
        mEditTextList.addView(getListItem("", index));
        // 获取刚刚添加的空列表项，并请求其内部的编辑文本视图获得焦点
        mEditTextList.getChildAt(index).findViewById(R.id.et_edit_text).requestFocus();

        // 隐藏笔记编辑器视图
        mNoteEditor.setVisibility(View.GONE);
        // 显示编辑文本列表视图
        mEditTextList.setVisibility(View.VISIBLE);
    }

    // 定义一个私有方法，用于获取高亮显示用户查询结果的可变字符串
    private Spannable getHighlightQueryResult(String fullText, String userQuery) {
        // 创建一个SpannableString对象，如果fullText为null，则使用空字符串
        SpannableString spannable = new SpannableString(fullText == null ? "" : fullText);
        // 检查用户查询字符串是否为空
        if (!TextUtils.isEmpty(userQuery)) {
            // 编译用户查询字符串为正则表达式模式
            mPattern = Pattern.compile(userQuery);
            // 创建一个匹配器，用于在fullText中查找匹配的用户查询字符串
            Matcher m = mPattern.matcher(fullText);
            // 初始化匹配的起始位置
            int start = 0;
            // 循环查找匹配的用户查询字符串
            while (m.find(start)) {
                // 在匹配的起始和结束位置之间设置背景颜色，以高亮显示查询结果
                spannable.setSpan(
                        // 创建一个BackgroundColorSpan对象，设置高亮背景颜色
                        new BackgroundColorSpan(this.getResources().getColor(
                                R.color.user_query_highlight)), m.start(), m.end(),
                        Spannable.SPAN_INCLUSIVE_EXCLUSIVE);
                start = m.end();
            }
        }
        return spannable;
    }

    // 定义一个私有方法，用于获取列表项的视图
    private View getListItem(String item, int index) {
        // 使用LayoutInflater从当前上下文中加载note_edit_list_item布局文件，并返回一个View对象
        View view = LayoutInflater.from(this).inflate(R.layout.note_edit_list_item, null);
        // 从视图中找到id为et_edit_text的NoteEditText控件
        final NoteEditText edit = (NoteEditText) view.findViewById(R.id.et_edit_text);
        // 设置edit文本的样式，使用TextAppearanceResources类获取对应的样式资源ID
        edit.setTextAppearance(this, TextAppearanceResources.getTexAppearanceResource(mFontSizeId));
        // 从视图中找到id为cb_edit_item的CheckBox控件
        CheckBox cb = ((CheckBox) view.findViewById(R.id.cb_edit_item));
        // 为CheckBox设置一个OnCheckedChangeListener监听器，用于处理选中状态变化
        cb.setOnCheckedChangeListener(new OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                // 如果CheckBox被选中
                if (isChecked) {
                    // 设置edit文本的画笔标志，添加删除线效果
                    edit.setPaintFlags(edit.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
                } else {
                    // 如果CheckBox未被选中，设置edit文本的画笔标志，去除删除线效果
                    edit.setPaintFlags(Paint.ANTI_ALIAS_FLAG | Paint.DEV_KERN_TEXT_FLAG);
                }
            }
        });

        // 检查item字符串是否以TAG_CHECKED开头
        if (item.startsWith(TAG_CHECKED)) {
            // 如果是，设置CheckBox为选中状态
            cb.setChecked(true);
            // 设置edit文本的画笔标志，添加删除线效果
            edit.setPaintFlags(edit.getPaintFlags() | Paint.STRIKE_THRU_TEXT_FLAG);
            // 去除TAG_CHECKED部分，并去除首尾空格
            item = item.substring(TAG_CHECKED.length(), item.length()).trim();
        } else if (item.startsWith(TAG_UNCHECKED)) {
            // 如果item字符串以TAG_UNCHECKED开头
            cb.setChecked(false);
            // 设置edit文本的画笔标志，去除删除线效果
            edit.setPaintFlags(Paint.ANTI_ALIAS_FLAG | Paint.DEV_KERN_TEXT_FLAG);
            // 去除TAG_UNCHECKED部分，并去除首尾空格
            item = item.substring(TAG_UNCHECKED.length(), item.length()).trim();
        }

        // 设置edit控件的文本内容，并高亮显示用户查询的关键词
        edit.setOnTextViewChangeListener(this);
        // 设置edit控件的索引
        edit.setIndex(index);
        // 设置edit控件的文本内容，并高亮显示用户查询的关键词
        edit.setText(getHighlightQueryResult(item, mUserQuery));
        // 返回加载的视图对象
        return view;
    }

    // 定义一个方法，当文本内容发生变化时调用
    public void onTextChange(int index, boolean hasText) {
        // 检查传入的索引是否超出子视图列表的范围
        if (index >= mEditTextList.getChildCount()) {
            // 如果索引超出范围，记录错误日志
            Log.e(TAG, "Wrong index, should not happen");
            // 结束方法执行
            return;
        }
        // 如果有文本内容
        if(hasText) {
            // 获取指定索引位置的子视图，并找到其中的 R.id.cb_edit_item 视图，设置为可见
            mEditTextList.getChildAt(index).findViewById(R.id.cb_edit_item).setVisibility(View.VISIBLE);
        } else {
            // 如果没有文本内容，获取指定索引位置的子视图，并找到其中的 R.id.cb_edit_item 视图，设置为不可见
            mEditTextList.getChildAt(index).findViewById(R.id.cb_edit_item).setVisibility(View.GONE);
        }
    }

    // 当检查列表模式改变时调用的方法
    public void onCheckListModeChanged(int oldMode, int newMode) {
        // 如果新的模式是检查列表模式
        if (newMode == TextNote.MODE_CHECK_LIST) {
            // 将编辑器的内容转换为列表模式
            switchToListMode(mNoteEditor.getText().toString());
        } else {
            // 如果获取工作文本失败
            if (!getWorkingText()) {

                // 将工作笔记的文本中的未选中标签移除
                mWorkingNote.setWorkingText(mWorkingNote.getContent().replace(TAG_UNCHECKED + " ", ""));
                        ""));
            }
            mNoteEditor.setText(getHighlightQueryResult(mWorkingNote.getContent(), mUserQuery));
            mEditTextList.setVisibility(View.GONE);
            mNoteEditor.setVisibility(View.VISIBLE);
        }
    }

    // 定义一个私有方法getWorkingText，用于获取工作文本并返回是否有已选中的项
    private boolean getWorkingText() {
        // 初始化一个布尔变量hasChecked，用于标记是否有已选中的项
        boolean hasChecked = false;
        // 检查当前工作笔记是否处于清单模式
        if (mWorkingNote.getCheckListMode() == TextNote.MODE_CHECK_LIST) {
            // 创建一个StringBuilder对象，用于构建工作文本
            StringBuilder sb = new StringBuilder();
            // 遍历mEditTextList中的所有子视图
            for (int i = 0; i < mEditTextList.getChildCount(); i++) {
                // 获取当前子视图
                View view = mEditTextList.getChildAt(i);
                // 从子视图中找到NoteEditText控件
                NoteEditText edit = (NoteEditText) view.findViewById(R.id.et_edit_text);
                // 检查edit文本是否为空
                if (!TextUtils.isEmpty(edit.getText())) {
                    // 检查子视图中的CheckBox是否被选中
                    if (((CheckBox) view.findViewById(R.id.cb_edit_item)).isChecked()) {
                        // 如果被选中，将选中标记和文本添加到StringBuilder中，并设置hasChecked为true
                        sb.append(TAG_CHECKED).append(" ").append(edit.getText()).append("\n");
                        hasChecked = true;
                    } else {
                        // 如果未被选中，将未选中标记和文本添加到StringBuilder中
                        sb.append(TAG_UNCHECKED).append(" ").append(edit.getText()).append("\n");
                    }
                }
            }
            // 将构建好的字符串设置为工作笔记的工作文本
            mWorkingNote.setWorkingText(sb.toString());
        } else {
            // 如果不是清单模式，直接将NoteEditor中的文本设置为工作笔记的工作文本
            mWorkingNote.setWorkingText(mNoteEditor.getText().toString());
        }
        // 返回是否有已选中的项
        return hasChecked;
    }

    // 定义一个私有方法saveNote，用于保存笔记
    private boolean saveNote() {
        // 调用getWorkingText方法获取当前正在编辑的文本内容
        getWorkingText();
        // 调用mWorkingNote对象的saveNote方法保存笔记，并将返回的布尔值赋给变量saved
        boolean saved = mWorkingNote.saveNote();
        // 如果笔记保存成功（saved为true）
        if (saved) {
            /**
             * There are two modes from List view to edit view, open one note,
             * create/edit a node. Opening node requires to the original
             * position in the list when back from edit view, while creating a
             * new node requires to the top of the list. This code
             * {@link #RESULT_OK} is used to identify the create/edit state
             */
            // 设置结果为RESULT_OK，表示操作成功
            setResult(RESULT_OK);
        }
        // 返回保存操作的结果
        return saved;
    }

    private void sendToDesktop() {
        /**
         * Before send message to home, we should make sure that current
         * editing note is exists in databases. So, for new note, firstly
         * save it
         */
        if (!mWorkingNote.existInDatabase()) {
            saveNote();
        }

        if (mWorkingNote.getNoteId() > 0) {
            Intent sender = new Intent();
            Intent shortcutIntent = new Intent(this, NoteEditActivity.class);
            shortcutIntent.setAction(Intent.ACTION_VIEW);
            shortcutIntent.putExtra(Intent.EXTRA_UID, mWorkingNote.getNoteId());
            sender.putExtra(Intent.EXTRA_SHORTCUT_INTENT, shortcutIntent);
            sender.putExtra(Intent.EXTRA_SHORTCUT_NAME,
                    makeShortcutIconTitle(mWorkingNote.getContent()));
            sender.putExtra(Intent.EXTRA_SHORTCUT_ICON_RESOURCE,
                    Intent.ShortcutIconResource.fromContext(this, R.drawable.icon_app));
            sender.putExtra("duplicate", true);
            sender.setAction("com.android.launcher.action.INSTALL_SHORTCUT");
            showToast(R.string.info_note_enter_desktop);
            sendBroadcast(sender);
        } else {
            /**
             * There is the condition that user has input nothing (the note is
             * not worthy saving), we have no note id, remind the user that he
             * should input something
             */
            Log.e(TAG, "Send to desktop error");
            showToast(R.string.error_note_empty_for_send_to_desktop);
        }
    }

    // 定义一个私有方法，用于生成快捷图标标题
    private String makeShortcutIconTitle(String content) {
        // 移除字符串中的已选中标签
        content = content.replace(TAG_CHECKED, "");
        // 移除字符串中的未选中标签
        content = content.replace(TAG_UNCHECKED, "");
        return content.length() > SHORTCUT_ICON_TITLE_MAX_LEN ? content.substring(0,
                SHORTCUT_ICON_TITLE_MAX_LEN) : content;
    }

    // 定义一个私有方法showToast，用于显示一个简短的Toast消息
    private void showToast(int resId) {
        // 调用另一个重载的showToast方法，传入resId和Toast.LENGTH_SHORT作为参数
        // resId是字符串资源的ID，Toast.LENGTH_SHORT表示Toast显示的时长为短时间
        showToast(resId, Toast.LENGTH_SHORT);
    }

    // 定义一个私有方法showToast，用于显示一个Toast消息
    private void showToast(int resId, int duration) {
        // 调用Toast类的静态方法makeText，创建一个Toast对象
        // this：当前上下文，通常是Activity
        // resId：要显示的消息字符串的资源ID
        // duration：Toast显示的时长，可以是Toast.LENGTH_SHORT或Toast.LENGTH_LONG
        Toast.makeText(this, resId, duration).show();
    }
}
