/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.graphics.Rect;
import android.text.Layout;
import android.text.Selection;
import android.text.Spanned;
import android.text.TextUtils;
import android.text.style.URLSpan;
import android.util.AttributeSet;
import android.util.Log;
import android.view.ContextMenu;
import android.view.KeyEvent;
import android.view.MenuItem;
import android.view.MenuItem.OnMenuItemClickListener;
import android.view.MotionEvent;
import android.widget.EditText;

import net.micode.notes.R;

import java.util.HashMap;
import java.util.Map;

public class NoteEditText extends EditText {
    // 日志标签，用于调试和日志输出
    private static final String TAG = "NoteEditText";
    // 当前编辑框的索引
    private int mIndex;
    // 删除操作前的光标位置
    private int mSelectionStartBeforeDelete;

    // 定义支持的链接方案
    private static final String SCHEME_TEL = "tel:" ;
    private static final String SCHEME_HTTP = "http:" ;
    private static final String SCHEME_EMAIL = "mailto:" ;

    // 链接方案与对应的操作资源ID映射
    private static final Map<String, Integer> sSchemaActionResMap = new HashMap<String, Integer>();
    static {
        // 初始化映射
        sSchemaActionResMap.put(SCHEME_TEL, R.string.note_link_tel);
        sSchemaActionResMap.put(SCHEME_HTTP, R.string.note_link_web);
        sSchemaActionResMap.put(SCHEME_EMAIL, R.string.note_link_email);
    }

    /**
     * Call by the {@link NoteEditActivity} to delete or add edit text
     */
    public interface OnTextViewChangeListener {
        /**
         * Delete current edit text when {@link KeyEvent#KEYCODE_DEL} happens
         * and the text is null
         */
        void onEditTextDelete(int index, String text);

        /**
         * Add edit text after current edit text when {@link KeyEvent#KEYCODE_ENTER}
         * happen
         */
        void onEditTextEnter(int index, String text);

        /**
         * Hide or show item option when text change
         */
        void onTextChange(int index, boolean hasText);
    }

    private OnTextViewChangeListener mOnTextViewChangeListener;

    // 构造函数，用于创建一个NoteEditText对象
    public NoteEditText(Context context) {
        // 调用父类的构造函数，传入当前上下文和null作为参数
        super(context, null);
        // 初始化成员变量mIndex，将其设置为0
        mIndex = 0;
    }

    // 定义一个公共方法，用于设置索引值
    public void setIndex(int index) {
        // 将传入的索引值赋值给成员变量mIndex
        mIndex = index;
    }

    // 定义一个方法用于设置TextView内容变化的监听器
    public void setOnTextViewChangeListener(OnTextViewChangeListener listener) {
        // 将传入的监听器对象赋值给成员变量mOnTextViewChangeListener
        mOnTextViewChangeListener = listener;
    }

    // 构造函数，用于创建NoteEditText对象
    public NoteEditText(Context context, AttributeSet attrs) {
        // 调用父类的构造函数，传入上下文对象和属性集，同时指定默认样式为android.R.attr.editTextStyle
        // 这里的super关键字表示调用父类（可能是TextView或EditText）的构造函数
        // context: 上下文对象，用于获取资源和系统服务
        // attrs: 属性集，包含在XML布局文件中定义的属性
        // android.R.attr.editTextStyle: 指定默认的EditText样式
        super(context, attrs, android.R.attr.editTextStyle);
    }

    // 构造函数，用于创建NoteEditText对象
    public NoteEditText(Context context, AttributeSet attrs, int defStyle) {
        // 调用父类（可能是TextView或EditText）的构造函数，传入上下文、属性集和样式资源ID
        super(context, attrs, defStyle);
        // TODO Auto-generated constructor stub
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        // 根据触摸事件的类型进行处理
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:

                // 获取触摸点的x坐标
                int x = (int) event.getX();
                // 获取触摸点的y坐标
                int y = (int) event.getY();
                // 调整x坐标，减去左侧的内边距
                x -= getTotalPaddingLeft();
                // 调整y坐标，减去顶部的内边距
                y -= getTotalPaddingTop();
                // 调整x坐标，加上视图的滚动偏移量
                x += getScrollX();
                // 调整y坐标，加上视图的滚动偏移量
                y += getScrollY();

                // 获取当前文本的布局
                Layout layout = getLayout();
                // 根据y坐标获取触摸点所在的行
                int line = layout.getLineForVertical(y);
                // 根据x坐标和行号获取触摸点在该行中的偏移量
                int off = layout.getOffsetForHorizontal(line, x);
                // 设置文本的选中位置为触摸点所在的偏移量
                Selection.setSelection(getText(), off);
                break;
        }

        // 调用父类的onTouchEvent方法，继续处理触摸事件
        return super.onTouchEvent(event);
    }

    @Override
    // 重写 onKeyDown 方法，用于处理按键按下事件
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        // 根据按键代码进行不同的处理
        switch (keyCode) {
            case KeyEvent.KEYCODE_ENTER:
                // 如果按键是回车键
                if (mOnTextViewChangeListener != null) {

                    // 如果存在文本视图内容变化监听器
                    return false; // 返回 false，表示不处理该按键事件
                }
                break; // 跳出 switch 语句
            case KeyEvent.KEYCODE_DEL:
                // 如果按键是删除键
                mSelectionStartBeforeDelete = getSelectionStart();

                // 记录删除操作前的光标位置
                break; // 跳出 switch 语句
            default:

                // 如果是其他按键
                break; // 跳出 switch 语句
        }
        // 调用父类的 onKeyDown 方法，处理其他按键事件
        return super.onKeyDown(keyCode, event);
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {
        // 根据按键代码进行不同的处理
        switch(keyCode) {
            case KeyEvent.KEYCODE_DEL: // 处理删除键
                if (mOnTextViewChangeListener != null) { // 检查是否有设置文本变化监听器
                    if (0 == mSelectionStartBeforeDelete && mIndex != 0) { // 检查删除位置是否在文本开始且不是第一个编辑框
                        // 调用监听器的删除方法，传递当前编辑框索引和文本内容
                        mOnTextViewChangeListener.onEditTextDelete(mIndex, getText().toString());
                        return true; // 返回true表示事件已处理
                    }
                } else {
                    // 如果没有设置监听器，记录日志
                    Log.d(TAG, "OnTextViewChangeListener was not seted");
                }
                break;
            case KeyEvent.KEYCODE_ENTER: // 处理回车键
                if (mOnTextViewChangeListener != null) { // 检查是否有设置文本变化监听器
                    int selectionStart = getSelectionStart(); // 获取当前光标位置
                    String text = getText().subSequence(selectionStart, length()).toString(); // 获取光标后的文本
                    setText(getText().subSequence(0, selectionStart)); // 清除光标后的文本
                    // 调用监听器的回车方法，传递下一个编辑框索引和光标后的文本内容
                    mOnTextViewChangeListener.onEditTextEnter(mIndex + 1, text);
                } else {
                    // 如果没有设置监听器，记录日志
                    Log.d(TAG, "OnTextViewChangeListener was not seted");
                }
                break;
            default: // 其他按键不做处理
                break;
        }
        // 调用父类方法处理其他按键事件
        return super.onKeyUp(keyCode, event);
    }

    @Override
    // 重写父类的方法，当视图焦点状态改变时调用
    protected void onFocusChanged(boolean focused, int direction, Rect previouslyFocusedRect) {
        // 检查是否有文本变化监听器
        if (mOnTextViewChangeListener != null) {
            // 如果当前视图失去焦点且文本为空
            if (!focused && TextUtils.isEmpty(getText())) {
                // 调用监听器的onTextChange方法，通知文本为空
                mOnTextViewChangeListener.onTextChange(mIndex, false);
            } else {
                // 否则，调用监听器的onTextChange方法，通知文本不为空
                mOnTextViewChangeListener.onTextChange(mIndex, true);
            }
        }
        // 调用父类的onFocusChanged方法，确保父类的逻辑也被执行
        super.onFocusChanged(focused, direction, previouslyFocusedRect);
    }

    @Override
    protected void onCreateContextMenu(ContextMenu menu) {
        // 检查当前文本是否为Spanned类型
        if (getText() instanceof Spanned) {
            // 获取当前选中的文本的起始和结束位置
            int selStart = getSelectionStart();
            int selEnd = getSelectionEnd();

            // 确定选中文本的起始和结束位置
            int min = Math.min(selStart, selEnd);
            int max = Math.max(selStart, selEnd);

            // 获取选中文本中的URLSpan对象数组
            final URLSpan[] urls = ((Spanned) getText()).getSpans(min, max, URLSpan.class);
            // 如果只有一个URLSpan对象
            if (urls.length == 1) {
                int defaultResId = 0;
                // 遍历预定义的URL模式映射表
                for(String schema: sSchemaActionResMap.keySet()) {
                    // 检查URL是否包含当前模式
                    if(urls[0].getURL().indexOf(schema) >= 0) {
                        // 如果匹配，获取对应的资源ID
                        defaultResId = sSchemaActionResMap.get(schema);
                        break;
                    }
                }

                // 如果没有匹配的模式，使用默认的资源ID
                if (defaultResId == 0) {
                    defaultResId = R.string.note_link_other;
                }

                // 向上下文菜单中添加一个菜单项
                menu.add(0, 0, 0, defaultResId).setOnMenuItemClickListener(
                        new OnMenuItemClickListener() {
                            public boolean onMenuItemClick(MenuItem item) {
                                // goto a new intent
                                urls[0].onClick(NoteEditText.this);
                                return true;
                            }
                        });
            }
        }
        super.onCreateContextMenu(menu);
    }
}
