/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.ui;

import android.content.Context;
import android.database.Cursor;
import android.text.TextUtils;

import net.micode.notes.data.Contact;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.tool.DataUtils;


public class NoteItemData {
    // 定义一个静态常量数组，用于查询数据库时使用的列名
    static final String [] PROJECTION = new String [] {
        NoteColumns.ID,                    // 笔记ID
        NoteColumns.ALERTED_DATE,          // 提醒日期
        NoteColumns.BG_COLOR_ID,           // 背景颜色ID
        NoteColumns.CREATED_DATE,          // 创建日期
        NoteColumns.HAS_ATTACHMENT,        // 是否有附件
        NoteColumns.MODIFIED_DATE,         // 修改日期
        NoteColumns.NOTES_COUNT,           // 笔记数量
        NoteColumns.PARENT_ID,             // 父ID
        NoteColumns.SNIPPET,               // 笔记摘要
        NoteColumns.TYPE,                  // 笔记类型
        NoteColumns.WIDGET_ID,             // 小部件ID
        NoteColumns.WIDGET_TYPE,           // 小部件类型
    };

    // 定义各个列在PROJECTION数组中的索引
    private static final int ID_COLUMN                    = 0;
    private static final int ALERTED_DATE_COLUMN          = 1;
    private static final int BG_COLOR_ID_COLUMN           = 2;
    private static final int CREATED_DATE_COLUMN          = 3;
    private static final int HAS_ATTACHMENT_COLUMN        = 4;
    private static final int MODIFIED_DATE_COLUMN         = 5;
    private static final int NOTES_COUNT_COLUMN           = 6;
    private static final int PARENT_ID_COLUMN             = 7;
    private static final int SNIPPET_COLUMN               = 8;
    private static final int TYPE_COLUMN                  = 9;
    private static final int WIDGET_ID_COLUMN             = 10;
    private static final int WIDGET_TYPE_COLUMN           = 11;

    // 定义笔记项的数据成员变量
    private long mId;
    private long mAlertDate;
    private int mBgColorId;
    private long mCreatedDate;
    private boolean mHasAttachment;
    private long mModifiedDate;
    private int mNotesCount;
    private long mParentId;
    private String mSnippet;
    private int mType;
    private int mWidgetId;
    private int mWidgetType;
    private String mName;
    private String mPhoneNumber;

    // 定义笔记项的位置相关标志
    private boolean mIsLastItem;
    private boolean mIsFirstItem;
    private boolean mIsOnlyOneItem;
    private boolean mIsOneNoteFollowingFolder;
    private boolean mIsMultiNotesFollowingFolder;

    // 构造函数，从Cursor中读取数据并初始化NoteItemData对象
    public NoteItemData(Context context, Cursor cursor) {
        mId = cursor.getLong(ID_COLUMN);
        mAlertDate = cursor.getLong(ALERTED_DATE_COLUMN);
        mBgColorId = cursor.getInt(BG_COLOR_ID_COLUMN);
        mCreatedDate = cursor.getLong(CREATED_DATE_COLUMN);
        mHasAttachment = (cursor.getInt(HAS_ATTACHMENT_COLUMN) > 0) ? true : false;
        mModifiedDate = cursor.getLong(MODIFIED_DATE_COLUMN);
        mNotesCount = cursor.getInt(NOTES_COUNT_COLUMN);
        mParentId = cursor.getLong(PARENT_ID_COLUMN);
        mSnippet = cursor.getString(SNIPPET_COLUMN);
        mSnippet = mSnippet.replace(NoteEditActivity.TAG_CHECKED, "").replace(
                NoteEditActivity.TAG_UNCHECKED, "");
        mType = cursor.getInt(TYPE_COLUMN);
        mWidgetId = cursor.getInt(WIDGET_ID_COLUMN);
        mWidgetType = cursor.getInt(WIDGET_TYPE_COLUMN);

        mPhoneNumber = "";
        // 如果父ID是通话记录文件夹，则获取通话号码和联系人名称
        if (mParentId == Notes.ID_CALL_RECORD_FOLDER) {
            mPhoneNumber = DataUtils.getCallNumberByNoteId(context.getContentResolver(), mId);
            if (!TextUtils.isEmpty(mPhoneNumber)) {
                mName = Contact.getContact(context, mPhoneNumber);
                if (mName == null) {
                    mName = mPhoneNumber;
                }
            }
        }

        if (mName == null) {
            mName = "";
        }
        // 检查笔记项的位置
        checkPostion(cursor);
    }

    // 检查笔记项在Cursor中的位置，并设置相应的标志
    private void checkPostion(Cursor cursor) {
        mIsLastItem = cursor.isLast() ? true : false;
        mIsFirstItem = cursor.isFirst() ? true : false;
        mIsOnlyOneItem = (cursor.getCount() == 1);
        mIsMultiNotesFollowingFolder = false;
        mIsOneNoteFollowingFolder = false;

        // 如果笔记类型是普通笔记且不是第一个笔记，则检查前一个笔记的类型
        if (mType == Notes.TYPE_NOTE && !mIsFirstItem) {
            int position = cursor.getPosition();
            if (cursor.moveToPrevious()) {
                if (cursor.getInt(TYPE_COLUMN) == Notes.TYPE_FOLDER
                        || cursor.getInt(TYPE_COLUMN) == Notes.TYPE_SYSTEM) {
                    if (cursor.getCount() > (position + 1)) {
                        mIsMultiNotesFollowingFolder = true;
                    } else {
                        mIsOneNoteFollowingFolder = true;
                    }
                }
                if (!cursor.moveToNext()) {
                    throw new IllegalStateException("cursor move to previous but can't move back");
                }
            }
        }
    }

    // 判断是否只有一个笔记跟随文件夹
    public boolean isOneFollowingFolder() {
        return mIsOneNoteFollowingFolder;
    }

    // 判断是否有多个笔记跟随文件夹
    public boolean isMultiFollowingFolder() {
        return mIsMultiNotesFollowingFolder;
    }

    // 判断是否是最后一个笔记项
    public boolean isLast() {
        return mIsLastItem;
    }

    // 获取通话记录的联系人名称
    public String getCallName() {
        return mName;
    }

    // 判断是否是第一个笔记项
    public boolean isFirst() {
        return mIsFirstItem;
    }

    // 判断是否只有一个笔记项
    public boolean isSingle() {
        return mIsOnlyOneItem;
    }

    // 获取笔记ID
    public long getId() {
        return mId;
    }

    // 获取提醒日期
    public long getAlertDate() {
        return mAlertDate;
    }

    // 获取创建日期
    public long getCreatedDate() {
        return mCreatedDate;
    }

    // 判断是否有附件
    public boolean hasAttachment() {
        return mHasAttachment;
    }

    // 获取修改日期
    public long getModifiedDate() {
        return mModifiedDate;
    }

    // 获取背景颜色ID
    public int getBgColorId() {
        return mBgColorId;
    }

    // 获取父ID
    public long getParentId() {
        return mParentId;
    }

    // 获取笔记数量
    public int getNotesCount() {
        return mNotesCount;
    }

    // 获取文件夹ID
    public long getFolderId () {
        return mParentId;
    }

    // 获取笔记类型
    public int getType() {
        return mType;
    }

    // 获取小部件类型
    public int getWidgetType() {
        return mWidgetType;
    }

    // 获取小部件ID
    public int getWidgetId() {
        return mWidgetId;
    }

    // 获取笔记摘要
    public String getSnippet() {
        return mSnippet;
    }

    // 判断是否有提醒
    public boolean hasAlert() {
        return (mAlertDate > 0);
    }

    // 判断是否是通话记录
    public boolean isCallRecord() {
        return (mParentId == Notes.ID_CALL_RECORD_FOLDER && !TextUtils.isEmpty(mPhoneNumber));
    }

    // 静态方法，从Cursor中获取笔记类型
    public static int getNoteType(Cursor cursor) {
        return cursor.getInt(TYPE_COLUMN);
    }
}
