/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.widget;
import android.app.PendingIntent;
import android.appwidget.AppWidgetManager;
import android.appwidget.AppWidgetProvider;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.util.Log;
import android.widget.RemoteViews;

import net.micode.notes.R;
import net.micode.notes.data.Notes;
import net.micode.notes.data.Notes.NoteColumns;
import net.micode.notes.tool.ResourceParser;
import net.micode.notes.ui.NoteEditActivity;
import net.micode.notes.ui.NotesListActivity;

public abstract class NoteWidgetProvider extends AppWidgetProvider {
    // 定义查询笔记信息的列
    public static final String [] PROJECTION = new String [] {
        NoteColumns.ID, // 笔记ID
        NoteColumns.BG_COLOR_ID, // 背景颜色ID
        NoteColumns.SNIPPET // 笔记摘要
    };



    // 定义列的索引
    public static final int COLUMN_ID           = 0; // 笔记ID列索引
    public static final int COLUMN_BG_COLOR_ID  = 1; // 背景颜色ID列索引
    public static final int COLUMN_SNIPPET      = 2; // 笔记摘要列索引

    // 日志标签
    private static final String TAG = "NoteWidgetProvider";

    @Override
    public void onDeleted(Context context, int[] appWidgetIds) {
        // 当小部件被删除时，更新数据库中的小部件ID为无效值
        ContentValues values = new ContentValues();
        values.put(NoteColumns.WIDGET_ID, AppWidgetManager.INVALID_APPWIDGET_ID);
        for (int i = 0; i < appWidgetIds.length; i++) {
            context.getContentResolver().update(Notes.CONTENT_NOTE_URI,
                    values,
                    NoteColumns.WIDGET_ID + "=?",
                    new String[] { String.valueOf(appWidgetIds[i])});
        }
    }

    // 获取指定小部件ID的笔记信息
    private Cursor getNoteWidgetInfo(Context context, int widgetId) {
        return context.getContentResolver().query(Notes.CONTENT_NOTE_URI,
                PROJECTION,
                NoteColumns.WIDGET_ID + "=? AND " + NoteColumns.PARENT_ID + "<>?",
                new String[] { String.valueOf(widgetId), String.valueOf(Notes.ID_TRASH_FOLER) },
                null);
    }

    // 更新小部件
    protected void update(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds) {
        update(context, appWidgetManager, appWidgetIds, false);
    }

    // 更新小部件，支持隐私模式
    // 更新应用小部件的方法
    private void update(Context context, AppWidgetManager appWidgetManager, int[] appWidgetIds,
            boolean privacyMode) {
        // 遍历所有的小部件ID
        for (int i = 0; i < appWidgetIds.length; i++) {
            // 检查小部件ID是否有效
            if (appWidgetIds[i] != AppWidgetManager.INVALID_APPWIDGET_ID) {
                int bgId = ResourceParser.getDefaultBgId(context); // 获取默认背景ID
                String snippet = ""; // 笔记摘要
                Intent intent = new Intent(context, NoteEditActivity.class);
                intent.setFlags(Intent.FLAG_ACTIVITY_SINGLE_TOP);
                intent.putExtra(Notes.INTENT_EXTRA_WIDGET_ID, appWidgetIds[i]);
                intent.putExtra(Notes.INTENT_EXTRA_WIDGET_TYPE, getWidgetType());

                // 获取小部件的笔记信息
                Cursor c = getNoteWidgetInfo(context, appWidgetIds[i]);
                if (c != null && c.moveToFirst()) {
                    // 检查是否有多个笔记使用相同的小部件ID
                    if (c.getCount() > 1) {
                        Log.e(TAG, "Multiple message with same widget id:" + appWidgetIds[i]);
                        c.close();
                        return;
                    }
                    snippet = c.getString(COLUMN_SNIPPET); // 获取笔记摘要
                    bgId = c.getInt(COLUMN_BG_COLOR_ID); // 获取背景颜色ID
                    intent.putExtra(Intent.EXTRA_UID, c.getLong(COLUMN_ID)); // 添加笔记ID
                    intent.setAction(Intent.ACTION_VIEW); // 设置意图为查看笔记
                } else {
                    // 如果没有找到笔记，显示默认提示信息
                    snippet = context.getResources().getString(R.string.widget_havenot_content);
                    intent.setAction(Intent.ACTION_INSERT_OR_EDIT); // 设置意图为插入或编辑笔记
                }

                if (c != null) {
                    c.close(); // 关闭游标
                }

                // 创建RemoteViews对象，用于更新小部件布局
                RemoteViews rv = new RemoteViews(context.getPackageName(), getLayoutId());
                rv.setImageViewResource(R.id.widget_bg_image, getBgResourceId(bgId)); // 设置背景图片
                intent.putExtra(Notes.INTENT_EXTRA_BACKGROUND_ID, bgId); // 添加背景ID到意图
                /**
                 * Generate the pending intent to start host for the widget
                 */
                PendingIntent pendingIntent = null;
                if (privacyMode) {
                    // 如果处于隐私模式，显示隐私提示信息
                    rv.setTextViewText(R.id.widget_text,
                            context.getString(R.string.widget_under_visit_mode));
                    // 创建PendingIntent，点击时启动NotesListActivity
                    pendingIntent = PendingIntent.getActivity(context, appWidgetIds[i], new Intent(
                            context, NotesListActivity.class), PendingIntent.FLAG_UPDATE_CURRENT);
                } else {
                    // 显示笔记摘要
                    rv.setTextViewText(R.id.widget_text, snippet);
                    // 创建PendingIntent，点击时启动NoteEditActivity
                    pendingIntent = PendingIntent.getActivity(context, appWidgetIds[i], intent,
                            PendingIntent.FLAG_UPDATE_CURRENT);
                }

                // 设置点击事件
                rv.setOnClickPendingIntent(R.id.widget_text, pendingIntent);
                // 更新小部件
                appWidgetManager.updateAppWidget(appWidgetIds[i], rv);
            }
        }
    }

    // 获取背景资源ID
    // 定义一个受保护的抽象方法，用于获取背景资源的ID
    // 该方法接受一个整数类型的参数bgId，表示背景资源的标识符
    // 返回值是一个整数，表示对应的背景资源ID
    protected abstract int getBgResourceId(int bgId);

    // 获取布局ID
    // 定义一个受保护的抽象方法 getLayoutId
    // 该方法返回一个整数，通常用于标识布局资源的ID
    protected abstract int getLayoutId();

    // 获取小部件类型
    // 定义一个受保护的抽象方法，用于获取小部件的类型
    // 该方法没有具体的实现，需要在子类中实现
    // 返回类型为int，表示小部件的类型
    protected abstract int getWidgetType();
}
